<?php
/**
 * Application Name: FlowMattic PHP Array
 * Description: Add PHP Array module to FlowMattic.
 * Version: 1.1
 * Author: InfiWebs
 * Author URI: https://www.infiwebs.com
 * Textdomain: flowmattic
 *
 * @package FlowMattic
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * PHP Array module integration class.
 *
 * @since 1.1
 */
class FlowMattic_Php_Array {
	/**
	 * Constructor.
	 *
	 * @access public
	 * @since 1.1
	 * @return void
	 */
	public function __construct() {
		// Enqueue custom view for filter.
		add_action( 'flowmattic_enqueue_views', array( $this, 'enqueue_views' ) );

		flowmattic_add_application(
			'php_array',
			array(
				'name'         => esc_attr__( 'PHP Array by FlowMattic', 'flowmattic' ),
				'icon'         => FLOWMATTIC_PLUGIN_URL . 'inc/apps/php-array/icon.svg',
				'instructions' => 'Perform array functions on the array data in your workflow',
				'actions'      => $this->get_actions(),
				'base'         => 'core',
				'type'         => 'action',
			)
		);
	}

	/**
	 * Enqueue view js.
	 *
	 * @access public
	 * @since 1.1
	 * @return void
	 */
	public function enqueue_views() {
		wp_enqueue_script( 'flowmattic-app-view-php-array', FLOWMATTIC_PLUGIN_URL . 'inc/apps/php-array/view-php-array.js', array( 'flowmattic-workflow-utils' ), FLOWMATTIC_VERSION, true );
	}

	/**
	 * Set actions.
	 *
	 * @access public
	 * @since 1.1
	 * @return array
	 */
	public function get_actions() {
		return array(
			'get_value_by_index' => array(
				'title'       => esc_attr__( 'Get Value by Index', 'flowmattic' ),
				'description' => esc_attr__( 'Get value of a specified index of an array', 'flowmattic' ),
			),
			'get_array_count'    => array(
				'title'       => esc_attr__( 'Get Array Count', 'flowmattic' ),
				'description' => esc_attr__( 'Get count of total number of items in array', 'flowmattic' ),
			),
			'array_search'       => array(
				'title'       => esc_attr__( 'Array Search', 'flowmattic' ),
				'description' => esc_attr__( 'Searches an array for a given value and returns the key', 'flowmattic' ),
			),
		);
	}

	/**
	 * Run the action step.
	 *
	 * @access public
	 * @since 1.1
	 * @param string $workflow_id  Workflow ID.
	 * @param object $step         Workflow current step.
	 * @param array  $capture_data Data captured by the WordPress action.
	 * @return array
	 */
	public function run_action_step( $workflow_id, $step, $capture_data ) {
		$action         = $step['action'];
		$fields         = isset( $step['fields'] ) ? $step['fields'] : ( isset( $step['actionAppArgs'] ) ? $step['actionAppArgs'] : array() );
		$json_data      = is_array( $fields['json_array'] ) ? $fields['json_array'] : ( is_array( json_decode( $fields['json_array'], true ) ) ? json_decode( $fields['json_array'], true ) : stripslashes( $fields['json_array'] ) );
		$json_array     = is_array( $json_data ) ? $json_data : json_decode( $json_data, true );
		$response_array = array();

		if ( is_array( $json_array ) ) {
			switch ( $action ) {
				case 'get_value_by_index':
					$array_index = explode( ',', $fields['array_index'] );
					$array_index = array_map( 'trim', $array_index );
					foreach ( $array_index as $key => $index ) {
						$index        = trim( $index );
						$array_value  = flowmattic_get_value_by_index( $json_array, $index );
						$simple_array = ( $array_value ) ? $array_value : array();

						if ( empty( $simple_array ) ) {
							$simple_array = array(
								'status'   => 'error',
								'response' => esc_html__( 'Array index not exits', 'flowmattic' ),
							);
						}

						if ( 1 === count( $array_index ) ) {
							if ( is_array( $simple_array ) ) {
								foreach ( $simple_array as $s_key => $s_value ) {
									$response_array[ $s_key ] = is_array( $s_value ) ? wp_json_encode( $s_value ) : $s_value;
								}
							} else {
								$response_array[ $index ] = $simple_array;
							}
						} else {
							$response_array[ $index ] = ( is_array( $simple_array ) ) ? wp_json_encode( $simple_array ) : $simple_array;
						}
					}

					$response = wp_json_encode( $response_array );
					break;

				case 'get_array_count':
					$count    = count( $json_array );
					$response = wp_json_encode(
						array(
							'status' => 'success',
							'count'  => $count,
						)
					);
					break;

				case 'array_search':
					$array_search_term = $fields['array_search_term'];

					$key = array_search( $array_search_term, $json_array );
					$key = ( 0 === $key ) ? '0' : $key;

					if ( ! $key ) {
						foreach ( $json_array as $k => $array_item ) {
							if ( is_array( $array_item ) ) {
								if ( 1 !== count( $array_item ) ) {
									foreach ( $array_item as $ar_index => $ar_item ) {
										if ( is_array( $ar_item ) ) {
											$key_2 = array_search( $array_search_term, $ar_item );
											$key_2 = ( 0 === $key_2 ) ? '0' : $key_2;

											if ( $key_2 ) {
												$key = ( 0 === $k ) ? '0/' . $ar_index . '/' . $key_2 : $k . '/' . $ar_index . '/' . $key_2;
												break;
											}
										} else {
											$key_2 = ( $array_search_term === $ar_item ) ? $ar_index : false;

											if ( $key_2 ) {
												$key = $k;
												break;
											}
										}
									}
								} else {
									$key_2 = array_search( $array_search_term, $array_item );
									$key_2 = ( 0 === $key_2 ) ? '0' : $key_2;
									if ( $key_2 ) {
										$key = ( 0 === $k ) ? '0' : $k;
										break;
									}
								}
							}
						}
					}

					if ( false !== $key ) {
						$response = wp_json_encode(
							array(
								'status' => 'success',
								'key'    => $key,
							)
						);
					} else {
						$response = wp_json_encode(
							array(
								'status'   => 'error',
								'response' => esc_html__( 'Nothing found for the given value', 'flowmattic' ),
							)
						);
					}
					break;
			}
		} else {
			$response = wp_json_encode(
				array(
					'status'  => 'error',
					'message' => esc_html__( 'Invalid JSON format.', 'flowmattic' ),
				)
			);
		}

		return $response;
	}

	/**
	 * Test action event ajax.
	 *
	 * @access public
	 * @since 1.1
	 * @param array $event_data Test event data.
	 * @return array
	 */
	public function test_event_action( $event_data ) {
		$event          = $event_data['event'];
		$fields         = isset( $event_data['fields'] ) ? $event_data['fields'] : ( isset( $settings['actionAppArgs'] ) ? $settings['actionAppArgs'] : array() );
		$workflow_id    = $event_data['workflow_id'];
		$response_array = array();

		$event_data['action'] = $event;

		$request = $this->run_action_step( $workflow_id, $event_data, $fields );

		return $request;
	}
}

new FlowMattic_Php_Array();
