<?php
/**
 * Application Name: WordPress
 * Description: Add WordPress integration to FlowMattic.
 * Version: 1.2
 * Author: InfiWebs
 * Author URI: https://www.infiwebs.com
 * Textdomain: flowmattic
 *
 * @package FlowMattic
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * WordPress integration class.
 *
 * @since 1.0
 */
class FlowMattic_Wordpress { // @codingStandardsIgnoreLine

	/**
	 * Variable to store if test event is running.
	 *
	 * @access protected
	 * @since 1.0
	 * @var bool
	 */
	protected $is_doing_test = false;

	/**
	 * Application name.
	 *
	 * @access protected
	 * @since 1.0
	 * @var bool
	 */
	protected $app_name = 'WordPress';

	/**
	 * Constructor.
	 *
	 * @access public
	 * @since 1.0
	 * @return void
	 */
	public function __construct() {
		// Enqueue custom view for WordPress.
		add_action( 'flowmattic_enqueue_views', array( $this, 'enqueue_views' ) );

		flowmattic_add_application(
			strtolower( $this->app_name ),
			array(
				'name'         => esc_attr__( 'WordPress', 'flowmattic' ),
				'icon'         => FLOWMATTIC_PLUGIN_URL . 'inc/apps/wordpress/icon.svg',
				'instructions' => 'WordPress is the world\'s most popular Content Management System.',
				'triggers'     => $this->get_triggers(),
				'actions'      => $this->get_actions(),
				'type'         => 'trigger, action',
			)
		);

		// Listen to WordPress user registration.
		add_action( 'user_register', array( $this, 'capture_user_registration' ), 10 );

		// Listen to WordPress user login.
		add_action( 'wp_login', array( $this, 'capture_user_login' ), 10, 2 );

		// Listen to new post creation.
		add_action( 'transition_post_status', array( $this, 'capture_post_publish' ), 10, 3 );

		// Listen to trash post action.
		add_action( 'draft_to_trash', array( $this, 'capture_post_trash' ), 10 );
		add_action( 'pending_to_trash', array( $this, 'capture_post_trash' ), 10 );
		add_action( 'publish_to_trash', array( $this, 'capture_post_trash' ), 10 );

		// Listen to existing post update.
		add_action( 'post_updated', array( $this, 'capture_post_update' ), 11, 3 );

		// Listen to new media upload.
		add_action( 'add_attachment', array( $this, 'capture_media_upload' ), 10 );

		// Listen to new comment post.
		add_action( 'comment_post', array( $this, 'capture_new_comment' ), 10, 3 );

		// Listen to comment approved.
		add_action( 'comment_approved', array( $this, 'capture_comment_approved' ), 10, 3 );

		// Listen to singular page view.
		add_action( 'wp_footer', array( $this, 'capture_singular_view' ), 99 );

		// Listen to post meta update.
		add_action( 'updated_post_meta', array( $this, 'capture_post_meta_update' ), 10, 4 );

		// Listen to user profile update.
		add_action( 'profile_update', array( $this, 'capture_user_profile_update' ), 10, 2 );

		// Listen to user password reset.
		add_action( 'password_reset', array( $this, 'capture_user_password_reset' ), 10, 2 );

		// Listen to user deletion.
		add_action( 'delete_user', array( $this, 'capture_deleted_user' ), 10, 3 );
	}

	/**
	 * Enqueue view js.
	 *
	 * @access public
	 * @since 1.0
	 * @return void
	 */
	public function enqueue_views() {
		wp_enqueue_script( 'flowmattic-app-view-wordpress', FLOWMATTIC_PLUGIN_URL . 'inc/apps/wordpress/view-wordpress.js', array( 'flowmattic-workflow-utils' ), FLOWMATTIC_VERSION, true );

		// Localize script for the flowmattic admin pages.
		wp_localize_script(
			'flowmattic-app-view-wordpress',
			'FMWPConfig',
			array(
				'user_roles'             => $this->get_all_user_roles(),
				'user_role_capabilities' => $this->get_all_user_role_capabilities(),
			)
		);
	}

	/**
	 * Set triggers.
	 *
	 * @access public
	 * @since 1.0
	 * @return array
	 */
	public function get_triggers() {
		return array(
			'new_user'             => array(
				'title'       => esc_attr__( 'New User', 'flowmattic' ),
				'description' => esc_attr__( 'Triggers when a new user is registered', 'flowmattic' ),
			),
			'new_post'             => array(
				'title'       => esc_attr__( 'New Post', 'flowmattic' ),
				'description' => esc_attr__( 'Triggers when a new post is published', 'flowmattic' ),
			),
			'new_media'            => array(
				'title'       => esc_attr__( 'New Media', 'flowmattic' ),
				'description' => esc_attr__( 'Triggers when a new media is uploaded', 'flowmattic' ),
			),
			'new_comment'          => array(
				'title'       => esc_attr__( 'New Comment', 'flowmattic' ),
				'description' => esc_attr__( 'Triggers when a new comment is added', 'flowmattic' ),
			),
			'comment_approved'     => array(
				'title'       => esc_attr__( 'Comment Approved', 'flowmattic' ),
				'description' => esc_attr__( 'Triggers when a comment is approved', 'flowmattic' ),
			),
			'post_updated'         => array(
				'title'       => esc_attr__( 'Update Post', 'flowmattic' ),
				'description' => esc_attr__( 'Triggers when a published post is updated', 'flowmattic' ),
			),
			'user_login'           => array(
				'title'       => esc_attr__( 'User Login', 'flowmattic' ),
				'description' => esc_attr__( 'Triggers when an user logs into the site', 'flowmattic' ),
			),
			'page_view'            => array(
				'title'       => esc_attr__( 'User Visits a Page/Post', 'flowmattic' ),
				'description' => esc_attr__( 'Triggers when an user visit page, post or any post type singular page.', 'flowmattic' ),
			),
			'trash_post'           => array(
				'title'       => esc_attr__( 'User Deletes a Post', 'flowmattic' ),
				'description' => esc_attr__( 'Triggers when an user delete post of any post type.', 'flowmattic' ),
			),
			'updated_post_meta'    => array(
				'title'       => esc_attr__( 'Post Meta Updated', 'flowmattic' ),
				'description' => esc_attr__( 'Triggers when post meta is updated', 'flowmattic' ),
			),
			'updated_user_profile' => array(
				'title'       => esc_attr__( 'User Profile Updated', 'flowmattic' ),
				'description' => esc_attr__( 'Triggers when user profile is updated', 'flowmattic' ),
			),
			'user_reset_password'  => array(
				'title'       => esc_attr__( 'User Resets Password', 'flowmattic' ),
				'description' => esc_attr__( 'Triggers when user resets the password', 'flowmattic' ),
			),
			'user_deleted'         => array(
				'title'       => esc_attr__( 'User Deleted', 'flowmattic' ),
				'description' => esc_attr__( 'Triggers when user is deleted from the database', 'flowmattic' ),
			),
		);
	}

	/**
	 * Set actions.
	 *
	 * @access public
	 * @since 1.0
	 * @return array
	 */
	public function get_actions() {
		return array(
			'new_user'               => array(
				'title'       => esc_attr__( 'New User', 'flowmattic' ),
				'description' => esc_attr__( 'Create a new user', 'flowmattic' ),
			),
			'update_user'            => array(
				'title'       => esc_attr__( 'Update User', 'flowmattic' ),
				'description' => esc_attr__( 'Update an existing user details', 'flowmattic' ),
			),
			'add_user_role'          => array(
				'title'       => esc_attr__( 'Add Role to User', 'flowmattic' ),
				'description' => esc_attr__( 'Add role to existing user roles of an user', 'flowmattic' ),
			),
			'change_user_role'       => array(
				'title'       => esc_attr__( 'Change User Role', 'flowmattic' ),
				'description' => esc_attr__( 'Change user role of an existing user and set new role', 'flowmattic' ),
			),
			'remove_user_role'       => array(
				'title'       => esc_attr__( 'Remove User Role', 'flowmattic' ),
				'description' => esc_attr__( 'Remove user role of an existing user', 'flowmattic' ),
			),
			'delete_user'            => array(
				'title'       => esc_attr__( 'Delete User', 'flowmattic' ),
				'description' => esc_attr__( 'Delete user account from site', 'flowmattic' ),
			),
			'new_comment'            => array(
				'title'       => esc_attr__( 'New Comment', 'flowmattic' ),
				'description' => esc_attr__( 'Create a new comment on post.', 'flowmattic' ),
			),
			'new_post'               => array(
				'title'       => esc_attr__( 'New Post', 'flowmattic' ),
				'description' => esc_attr__( 'Create a new post', 'flowmattic' ),
			),
			'update_post'            => array(
				'title'       => esc_attr__( 'Update Post', 'flowmattic' ),
				'description' => esc_attr__( 'Update existing post of any post type with custom fields', 'flowmattic' ),
			),
			'new_media'              => array(
				'title'       => esc_attr__( 'New Media', 'flowmattic' ),
				'description' => esc_attr__( 'Create a new media', 'flowmattic' ),
			),
			'update_user_meta'       => array(
				'title'       => esc_attr__( 'Update User Meta', 'flowmattic' ),
				'description' => esc_attr__( 'Update User Metadata', 'flowmattic' ),
			),
			'get_user_by_id'         => array(
				'title'       => esc_attr__( 'Get User by ID', 'flowmattic' ),
				'description' => esc_attr__( 'Get an user profile by ID', 'flowmattic' ),
			),
			'get_user_by_email'      => array(
				'title'       => esc_attr__( 'Get User by Email', 'flowmattic' ),
				'description' => esc_attr__( 'Get an user profile by Email', 'flowmattic' ),
			),
			'create_category'        => array(
				'title'       => esc_attr__( 'Create Category', 'flowmattic' ),
				'description' => esc_attr__( 'Create a new category', 'flowmattic' ),
			),
			'create_tag'             => array(
				'title'       => esc_attr__( 'Create Tag', 'flowmattic' ),
				'description' => esc_attr__( 'Create a new post tag', 'flowmattic' ),
			),
			'add_role'               => array(
				'title'       => esc_attr__( 'Create Role', 'flowmattic' ),
				'description' => esc_attr__( 'Create a new user role, if not exists', 'flowmattic' ),
			),
			'get_post_meta'          => array(
				'title'       => esc_attr__( 'Get Post Metadata', 'flowmattic' ),
				'description' => esc_attr__( 'Retrieve all or custom metadata from any post', 'flowmattic' ),
			),
			'get_user_meta'          => array(
				'title'       => esc_attr__( 'Get User Metadata', 'flowmattic' ),
				'description' => esc_attr__( 'Retrieve all or custom metadata from any user', 'flowmattic' ),
			),
			'get_posts_by_post_type' => array(
				'title'       => esc_attr__( 'Get Posts by Post Type', 'flowmattic' ),
				'description' => esc_attr__( 'Retrieve all posts of specific post type', 'flowmattic' ),
			),
			'get_posts_by_meta'      => array(
				'title'       => esc_attr__( 'Get Posts by Metadata', 'flowmattic' ),
				'description' => esc_attr__( 'Retrieve all posts of specific post type by metadata or custom fields', 'flowmattic' ),
			),
		);
	}

	/**
	 * Check if live capturing is in process for this application.
	 *
	 * @access public
	 * @since 1.0
	 * @return bool|string False if not this application, workflow id if current application.
	 */
	public function is_capturing() {
		$workflow_id   = get_option( 'webhook-capture-live', false );
		$workflow_name = get_option( 'webhook-capture-application', false );

		if ( strtolower( $this->app_name ) !== $workflow_name ) {
			return false;
		} else {
			delete_option( 'webhook-capture-live' );
			delete_option( 'webhook-capture-application' );
			return $workflow_id;
		}
	}

	/**
	 * Run the workflow step.
	 *
	 * @access public
	 * @since 1.0
	 * @param string $action       WordPress action.
	 * @param array  $capture_data Data captured by the WordPress action.
	 * @return array
	 */
	public function run_workflow_step( $action, $capture_data ) {

		// If doing test, skip the workflow execution.
		if ( $this->is_doing_test ) {
			return false;
		}

		// Get the workflows having WordPress as trigger.
		$workflows = wp_flowmattic()->workflows_db->get_workflow_by_trigger_application( strtolower( $this->app_name ) );

		if ( ! empty( $workflows ) ) {
			foreach ( $workflows as $key => $workflow ) {
				$workflow_steps  = json_decode( $workflow->workflow_steps );
				$settings        = json_decode( $workflow->workflow_settings, true );
				$workflow_status = ( isset( $settings['status'] ) && $settings['status'] ) ? $settings['status'] : 'off';

				// Do not execute workflows that are not turned ON.
				if ( 'on' !== $workflow_status ) {
					continue;
				}

				// Loop through each workflow to check the action event.
				if ( ! empty( $workflow_steps ) ) {
					foreach ( $workflow_steps as $step ) {
						if ( 'trigger' === $step->type && strtolower( $this->app_name ) === $step->application ) {
							$workflow_id = $workflow->workflow_id;

							if ( 'page_view' === $action ) {
								$trigger_post_id = ( isset( $step->trigger_post_id ) && '' !== $step->trigger_post_id ) ? $step->trigger_post_id : '';

								if ( '' !== $trigger_post_id && (int) $capture_data['id'] !== (int) $trigger_post_id ) {
									continue;
								}
							}

							if ( $action === $step->action ) {
								// Run the workflow.
								$flowmattic_workflow = new FlowMattic_Workflow();
								$flowmattic_workflow->run( $workflow_id, $capture_data );

								break;
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Run the action step.
	 *
	 * @access public
	 * @since 1.0
	 * @param string $workflow_id  Workflow ID.
	 * @param object $step         Workflow current step.
	 * @param array  $capture_data Data captured by the WordPress action.
	 * @return array
	 */
	public function run_action_step( $workflow_id, $step, $capture_data ) {

		$step   = (array) $step;
		$action = $step['action'];

		switch ( $action ) {
			case 'new_user':
				$response = $this->create_new_user( $step );
				break;

			case 'update_user':
				$response = $this->update_existing_user( $step );
				break;

			case 'new_media':
				$response = $this->create_new_media( $step );
				break;

			case 'new_comment':
				$response = $this->create_new_comment( $step );
				break;

			case 'new_post':
				$response = $this->create_new_post( $step );
				break;

			case 'update_post':
				$response = $this->update_post( $step );
				break;

			case 'update_user_meta':
				$fields = $step;

				if ( isset( $step['actionAppArgs']['user_id'] ) ) {
					$fields['user_id'] = $step['actionAppArgs']['user_id'];
				}

				$fields['meta_fields'] = isset( $step['meta_fields'] ) ? $step['meta_fields'] : array();

				$response = $this->update_user_meta( $fields );
				break;

			case 'get_user_by_id':
				$fields = $step;

				if ( isset( $step['actionAppArgs']['user_id'] ) ) {
					$fields['user_id'] = $step['actionAppArgs']['user_id'];
				}

				$response = $this->get_user_by_id( $fields );
				break;

			case 'get_user_by_email':
				$fields = $step;

				if ( isset( $step['actionAppArgs']['user_email'] ) ) {
					$fields['user_email'] = $step['actionAppArgs']['user_email'];
				}

				$response = $this->get_user_by_email( $fields );
				break;

			case 'create_category':
				$fields = $step;

				if ( isset( $step['actionAppArgs']['category_name'] ) ) {
					$fields['category_name'] = $step['actionAppArgs']['category_name'];
				}

				$response = $this->create_category( $fields );
				break;

			case 'create_tag':
				$fields = $step;

				if ( isset( $step['actionAppArgs']['tag_name'] ) ) {
					$fields['tag_name'] = $step['actionAppArgs']['tag_name'];
				}

				$response = $this->create_tag( $fields );
				break;

			case 'add_user_role':
				$response = $this->add_user_role( $step );
				break;

			case 'change_user_role':
				$response = $this->change_user_role( $step );
				break;

			case 'remove_user_role':
				$response = $this->remove_user_role( $step );
				break;

			case 'delete_user':
				$response = $this->delete_user( $step );
				break;

			case 'add_role':
				$fields = $step;

				if ( isset( $step['actionAppArgs'] ) ) {
					$fields = $step['actionAppArgs'];
				}

				$response = $this->add_role( $fields );
				break;

			case 'get_post_meta':
				$response = $this->get_post_meta( $step );
				break;

			case 'get_user_meta':
				$response = $this->get_user_meta( $step );
				break;

			case 'get_posts_by_meta':
				$response = $this->get_posts_by_meta( $step );
				break;

			case 'get_posts_by_post_type':
				$response = $this->get_posts_by_post_type( $step );
				break;
		}

		return $response;
	}

	/**
	 * Action fires after WordPress user is registered.
	 *
	 * @access public
	 * @since 1.0
	 * @param int $user_id Registered user ID.
	 * @return bool
	 */
	public function capture_user_registration( $user_id ) {
		$workflow_executed = get_user_meta( $user_id, 'workflow_executed', true );

		if ( '1' === $workflow_executed ) {
			return false;
		}

		$workflow_id = $this->is_capturing();

		$user_data = flowmattic_get_user_data( $user_id );

		// Remove the user passwords from the received data for privacy concern.
		unset( $user_data['pass1'] );
		unset( $user_data['pass2'] );
		unset( $user_data['password'] );
		unset( $user_data['user_pass'] );
		unset( $user_data['userpass'] );
		unset( $user_data['user_password'] );

		$user_data['user_id'] = $user_id;

		// If workflow ID is set, the webhook capture event is in progress.
		if ( $workflow_id ) {
			$trigger_event = get_option( 'webhook-capture-app-action', false );
			if ( $trigger_event && 'new_user' === $trigger_event ) {
				update_option( 'webhook-capture-' . $workflow_id, $user_data, false );
			}
		}

		// Set workflow executed flag.
		update_user_meta( $user_id, 'workflow_executed', '1' );

		// Get all available meta data for user.
		$user_meta = $this->get_user_metadata( $user_id );

		// Run the workflow.
		$this->run_workflow_step( 'new_user', $user_data );
	}

	/**
	 * Action fires after new post is published.
	 *
	 * @access public
	 * @since 1.0
	 * @param string  $new_status Transition to this post status.
	 * @param string  $old_status Previous post status.
	 * @param WP_Post $new_post   Post data.
	 * @return void
	 */
	public function capture_post_publish( $new_status, $old_status, $new_post ) {
		// Bail, if already published post is being updated.
		if ( 'publish' === $old_status ) {
			return;
		}

		// Bail, if current post is not being published.
		if ( 'publish' !== $new_status ) {
			return;
		}

		// Get the workflow executed flag.
		$is_workflow_executed = get_post_meta( $new_post->ID, 'workflow_executed', true );

		if ( 'yes' === $is_workflow_executed ) {
			return;
		}

		// Set the workflow executed flag for this order.
		update_post_meta( $new_post->ID, 'workflow_executed', 'yes' );

		// Continue if post is published.
		$workflow_id = $this->is_capturing();
		$post_data   = (array) $new_post;

		$current_taxonomies = get_object_taxonomies( $new_post, 'objects' );
		$taxonomies         = array();

		foreach ( $current_taxonomies as $tax_title => $tax ) {
			$terms = get_the_terms( $new_post, $tax_title );
			if ( $terms ) {
				foreach ( $terms as $key => $term ) {
					$taxonomies[ $tax_title ][] = array(
						'name'    => $term->name,
						'slug'    => $term->slug,
						'term_id' => $term->term_id,
					);
				}
			}
		}

		$post_data['taxonomies'] = wp_json_encode( $taxonomies );

		// If workflow ID is set, the webhook capture event is in progress.
		if ( $workflow_id ) {
			$trigger_event = get_option( 'webhook-capture-app-action', false );
			if ( $trigger_event && 'new_post' === $trigger_event ) {
				update_option( 'webhook-capture-' . $workflow_id, $post_data, false );
			}
		}

		// Run the workflow.
		$this->run_workflow_step( 'new_post', $post_data );
	}

	/**
	 * Action fires after post is trashed.
	 *
	 * @access public
	 * @since 1.2
	 * @param WP_Post $post Post object.
	 * @return void
	 */
	public function capture_post_trash( $post ) {
		// Continue if post is published.
		$workflow_id = $this->is_capturing();
		$post_data   = (array) $post;

		$current_taxonomies = get_object_taxonomies( $post, 'objects' );
		$taxonomies         = array();

		foreach ( $current_taxonomies as $tax_title => $tax ) {
			$terms = get_the_terms( $post, $tax_title );
			if ( $terms ) {
				foreach ( $terms as $key => $term ) {
					$taxonomies[ $tax_title ][] = array(
						'name'    => $term->name,
						'slug'    => $term->slug,
						'term_id' => $term->term_id,
					);
				}
			}
		}

		$post_data['taxonomies'] = wp_json_encode( $taxonomies );

		// User who trashed this post.
		$current_user                   = wp_get_current_user();
		$post_data['user_name']         = esc_html( $current_user->user_login );
		$post_data['user_email']        = esc_html( $current_user->user_email );
		$post_data['user_first_name']   = esc_html( $current_user->user_firstname );
		$post_data['user_last_name']    = esc_html( $current_user->user_lastname );
		$post_data['user_display_name'] = esc_html( $current_user->display_name );
		$post_data['user_ID']           = esc_html( $current_user->ID );

		// If workflow ID is set, the webhook capture event is in progress.
		if ( $workflow_id ) {
			$trigger_event = get_option( 'webhook-capture-app-action', false );
			if ( $trigger_event && 'trash_post' === $trigger_event ) {
				update_option( 'webhook-capture-' . $workflow_id, $post_data, false );
			}
		}

		// Run the workflow.
		$this->run_workflow_step( 'trash_post', $post_data );
	}

	/**
	 * Action fires after existing post is updated.
	 *
	 * @access public
	 * @since 1.0
	 * @param int     $post_id     ID of the post being updated.
	 * @param WP_Post $post_after  Post object.
	 * @param bool    $post_before Post object before update.
	 * @return void
	 */
	public function capture_post_update( $post_id, $post_after, $post_before ) {
		// Continue if existing post is being updated.
		$workflow_id = $this->is_capturing();
		$post_data   = (array) $post_after;

		// Get the workflow executed flag.
		$workflow_executed_at = get_post_meta( $post_after->ID, 'workflow_executed_at', time() );

		if ( time() <= $workflow_executed_at ) {
			return;
		}

		// Set the workflow executed flag for this order.
		update_post_meta( $post_after->ID, 'workflow_executed_at', time() + 5 );

		$current_taxonomies = get_object_taxonomies( $post_after, 'objects' );
		$taxonomies         = array();

		foreach ( $current_taxonomies as $tax_title => $tax ) {
			$terms = get_the_terms( $post_after, $tax_title );
			if ( $terms ) {
				foreach ( $terms as $key => $term ) {
					$taxonomies[ $tax_title ][] = array(
						'name'    => $term->name,
						'slug'    => $term->slug,
						'term_id' => $term->term_id,
					);
				}
			}
		}

		$post_data['taxonomies'] = wp_json_encode( $taxonomies );

		// If workflow ID is set, the webhook capture event is in progress.
		if ( $workflow_id ) {
			$trigger_event = get_option( 'webhook-capture-app-action', false );
			if ( $trigger_event && 'post_updated' === $trigger_event ) {
				update_option( 'webhook-capture-' . $workflow_id, $post_data, false );
			}
		}

		// Run the workflow.
		$this->run_workflow_step( 'post_updated', $post_data );
	}

	/**
	 * Action fires after media is uploaded.
	 *
	 * @access public
	 * @since 1.0
	 * @param int $post_id Attachment ID.
	 * @return void
	 */
	public function capture_media_upload( $post_id ) {
		$media = flowmattic_get_attachment( $post_id );

		$workflow_id = $this->is_capturing();

		$post_data = (array) $media;

		// If workflow ID is set, the webhook capture event is in progress.
		if ( $workflow_id ) {
			update_option( 'webhook-capture-' . $workflow_id, $post_data, false );
		}

		// Run the workflow.
		$this->run_workflow_step( 'new_media', $post_data );
	}

	/**
	 * Action fires after new comment is inserted into the database.
	 *
	 * @access public
	 * @since 1.0
	 * @param int        $comment_id       Comment ID.
	 * @param int|string $comment_approved If comment is approved or is spam.
	 * @param array      $comment_data     Comment data.
	 * @return void
	 */
	public function capture_new_comment( $comment_id, $comment_approved, $comment_data ) {
		$commented_post = get_post( $comment_data['comment_post_ID'] );
		$workflow_id    = $this->is_capturing();

		$post_data = $comment_data;

		$current_taxonomies = get_object_taxonomies( $commented_post, 'objects' );
		$taxonomies         = array();

		foreach ( $current_taxonomies as $tax_title => $tax ) {
			$terms = get_the_terms( $commented_post, $tax_title );
			if ( $terms ) {
				foreach ( $terms as $key => $term ) {
					$taxonomies[ $tax_title ][] = array(
						'name'    => $term->name,
						'slug'    => $term->slug,
						'term_id' => $term->term_id,
					);
				}
			}
		}

		$post_data['taxonomies']                = wp_json_encode( $taxonomies );
		$post_data['commented_posts_post_type'] = get_post_type( $commented_post );

		// If workflow ID is set, the webhook capture event is in progress.
		if ( $workflow_id ) {
			update_option( 'webhook-capture-' . $workflow_id, $post_data, false );
		}

		// Run the workflow.
		$this->run_workflow_step( 'new_comment', $post_data );
	}

	/**
	 * Action fires after comment approved into the database.
	 *
	 * @access public
	 * @since 1.0
	 * @param int        $comment    Comment.
	 * @param int|string $new_status If comment is approved or is spam.
	 * @param array      $old_status Comment data.
	 * @return void
	 */
	public function capture_comment_approved( $comment, $new_status, $old_status ) {
		if ( 'approved' !== $new_status ) {
			return;
		}

		$commented_post = get_post( $comment->comment_post_ID );

		$post_data = array(
			'comment_id'   => $comment->comment_ID,
			'comment_post' => $commented_post,
		);

		$this->run_workflow_step( 'comment_approved', $post_data );
	}

	/**
	 * Capture singular view.
	 *
	 * @access public
	 * @since 1.0
	 * @return void
	 */
	public function capture_singular_view() {
		global $post;

		// Continue if post is published.
		$workflow_id = $this->is_capturing();

		if ( $workflow_id ) {
			$trigger_event = get_option( 'webhook-capture-app-action', false );
			if ( $trigger_event && 'page_view' !== $trigger_event ) {
				return;
			}

			if ( ! is_singular() ) {
				return;
			}
		} elseif ( ! get_option( 'wp_page_view_trigger', false ) ) {
				return;
		}

		$current_user = ( is_user_logged_in() ) ? wp_get_current_user() : 'anonymous';
		$user_info    = array();
		$post_data    = array();

		if ( 'anonymous' !== $current_user ) {
			$post_data['user_name']         = esc_html( $current_user->user_login );
			$post_data['user_email']        = esc_html( $current_user->user_email );
			$post_data['user_first_name']   = esc_html( $current_user->user_firstname );
			$post_data['user_last_name']    = esc_html( $current_user->user_lastname );
			$post_data['user_display_name'] = esc_html( $current_user->display_name );
			$post_data['user_ID']           = esc_html( $current_user->ID );
		} else {
			$post_data['user'] = $current_user;
		}

		$current_taxonomies = get_object_taxonomies( $post, 'objects' );
		$taxonomies         = array();

		foreach ( $current_taxonomies as $tax_title => $tax ) {
			$terms = get_the_terms( $post, $tax_title );
			if ( $terms ) {
				foreach ( $terms as $key => $term ) {
					$taxonomies[ $tax_title ][] = array(
						'name'    => $term->name,
						'slug'    => $term->slug,
						'term_id' => $term->term_id,
					);
				}
			}
		}

		$post_data['post_type']    = get_post_type();
		$post_data['id']           = get_the_ID();
		$post_data['post_title']   = get_the_title();
		$post_data['taxonomies']   = wp_json_encode( $taxonomies );
		$post_data['http_referer'] = wp_get_referer();

		// Check if URL has any parameters.
		if ( ! empty( $_GET ) ) {
			$url_params     = $_GET;
			$url_param_data = array();

			foreach ( $url_params as $key => $value ) {
				$key = 'url_param_' . $key;

				if ( is_array( $value ) || is_object( $value ) ) {
					$url_param_data = flowmattic_recursive_array( $url_param_data, $key, $value );
				} else {
					$url_param_data[ $key ] = $value;
				}
			}

			$post_data = array_merge( $post_data, $url_param_data );
		}

		// If workflow ID is set, the webhook capture event is in progress.
		if ( $workflow_id ) {
			$args           = array(
				'workflow_id' => $workflow_id,
			);
			$workflow       = wp_flowmattic()->workflows_db->get( $args );
			$workflow_steps = json_decode( $workflow->workflow_steps, true );
			$settings       = $workflow_steps[0];

			// Prevent from executing workflow in test mode if not WordPress trigger.
			if ( strtolower( $this->app_name ) !== $settings['application'] ) {
				return;
			}

			$trigger_post_id = ( isset( $settings['trigger_post_id'] ) && '' !== $settings['trigger_post_id'] ) ? $settings['trigger_post_id'] : '';

			if ( '' !== $trigger_post_id && (int) $post_data['id'] !== (int) $trigger_post_id ) {
				return;
			}

			update_option( 'webhook-capture-' . $workflow_id, $post_data, false );
		}

		// Run the workflow.
		$this->run_workflow_step( 'page_view', $post_data );
	}

	/**
	 * Capture post meta update.
	 *
	 * @access public
	 * @since 1.2
	 * @param int    $meta_id     ID of updated metadata entry.
	 * @param int    $object_id   ID of the object metadata is for.
	 * @param string $meta_key    Metadata key.
	 * @param mixed  $_meta_value Metadata value.
	 * @return void
	 */
	public function capture_post_meta_update( $meta_id, $object_id, $meta_key, $_meta_value ) {
		if ( '_edit_lock' === $meta_key ) {
			return;
		}

		// Continue if post is published.
		$workflow_id = $this->is_capturing();
		$post_data   = array(
			'meta_key'   => $meta_key, // @codingStandardsIgnoreLine
			'meta_value' => $_meta_value, // @codingStandardsIgnoreLine
		);

		// If workflow ID is set, the webhook capture event is in progress.
		if ( $workflow_id ) {
			$trigger_event = get_option( 'webhook-capture-app-action', false );
			if ( $trigger_event && 'updated_post_meta' === $trigger_event ) {
				update_option( 'webhook-capture-' . $workflow_id, $post_data, false );

				// Do not execute workflow if capture data in process.
				return;
			}
		}

		// Run the workflow.
		$this->run_workflow_step( 'updated_post_meta', $post_data );
	}

	/**
	 * Capture user profile update.
	 *
	 * @access public
	 * @since 1.2
	 * @param int    $user_id       ID of updated metadata entry.
	 * @param Object $old_user_data User data before update.
	 * @return bool
	 */
	public function capture_user_profile_update( $user_id, $old_user_data ) {
		$workflow_id = get_option( 'webhook-capture-live', false );
		$app_action  = get_option( 'webhook-capture-app-action', false );
		$post_data   = array();

		$user      = get_userdata( $user_id );
		$post_data = (array) $user->data;

		$user_meta = $this->get_user_metadata( $user_id );

		foreach ( $user_meta as $key => $value ) {
			$post_data[ $key ] = $value;
		}

		// Remove the user passwords from the received data for privacy concern.
		unset( $post_data['password'] );
		unset( $post_data['user_pass'] );
		unset( $post_data['userpass'] );
		unset( $post_data['user_password'] );

		// If workflow ID is set, the webhook capture event is in progress.
		if ( $workflow_id && 'updated_user_profile' !== $app_action ) {
			return false;
		}

		if ( $workflow_id ) {
			update_option( 'webhook-capture-' . $workflow_id, $post_data, false );
			delete_option( 'webhook-capture-live' );

			// Do not execute workflow if capture data in process.
			return;
		}

		// Run the workflow.
		$this->run_workflow_step( 'updated_user_profile', $post_data );
	}

	/**
	 * Capture user password reset.
	 *
	 * @access public
	 * @since 1.2
	 * @param Object $user     User resetting the password.
	 * @param string $new_pass New password reset by user.
	 * @return void
	 */
	public function capture_user_password_reset( $user, $new_pass ) {
		// Continue if post is published.
		$workflow_id = $this->is_capturing();
		$post_data   = (array) $user->data;

		// Remove the user passwords from the received data for privacy concern.
		unset( $post_data['password'] );
		unset( $post_data['user_pass'] );
		unset( $post_data['userpass'] );
		unset( $post_data['user_password'] );

		// If workflow ID is set, the webhook capture event is in progress.
		if ( $workflow_id ) {
			$trigger_event = get_option( 'webhook-capture-app-action', false );
			if ( $trigger_event && 'user_reset_password' === $trigger_event ) {
				update_option( 'webhook-capture-' . $workflow_id, $post_data, false );
			}
		}

		// Run the workflow.
		$this->run_workflow_step( 'user_reset_password', $post_data );
	}

	/**
	 * Capture deleted user.
	 *
	 * @access public
	 * @since 1.2
	 * @param int      $id       Deleted user ID.
	 * @param int|null $reassign ID of the user reassigned.
	 * @param Object   $user     Deleted user object.
	 * @return void
	 */
	public function capture_deleted_user( $id, $reassign, $user ) {
		// Continue if post is published.
		$workflow_id = $this->is_capturing();
		$post_data   = (array) $user->data;

		// Remove the user passwords from the received data for privacy concern.
		unset( $post_data['password'] );
		unset( $post_data['user_pass'] );
		unset( $post_data['userpass'] );
		unset( $post_data['user_password'] );

		$user_meta = $this->get_user_metadata( $id );
		$post_data = array_merge( $post_data, $user_meta );

		// If workflow ID is set, the webhook capture event is in progress.
		if ( $workflow_id ) {
			$trigger_event = get_option( 'webhook-capture-app-action', false );
			if ( $trigger_event && 'user_deleted' === $trigger_event ) {
				update_option( 'webhook-capture-' . $workflow_id, $post_data, false );
			}
		}

		// Run the workflow.
		$this->run_workflow_step( 'user_deleted', $post_data );
	}

	/**
	 * Create new user.
	 *
	 * @access public
	 * @since 1.0
	 * @param array $data Request data.
	 * @return array
	 */
	public function create_new_user( $data ) {
		$response = array();

		$user_roles = isset( $data['user_role'] ) ? explode( ',', $data['user_role'] ) : 'subscriber';

		$password = wp_generate_password();

		if ( isset( $data['auto_password_generation'] ) && 'Yes' === $data['auto_password_generation'] ) {
			$password = wp_generate_password();
		} elseif ( isset( $data['password'] ) && '' !== $data['password'] ) {
			$password = $data['password'];
		}

		$user_data = array(
			'user_login' => $data['username'],
			'user_pass'  => $password,
			'user_email' => $data['email'],
			'first_name' => isset( $data['first_name'] ) ? $data['first_name'] : '',
			'last_name'  => isset( $data['last_name'] ) ? $data['last_name'] : '',
			'role'       => is_array( $user_roles ) ? $user_roles[0] : $user_roles,
		);

		$notify = ( isset( $data['notification_type'] ) && 'none' !== $data['notification_type'] ) ? $data['notification_type'] : false;

		$user = get_user_by( 'email', $data['email'] );

		if ( ! $user ) {
			$user_id = wp_insert_user( $user_data );
		} else {
			$user_id = $user->ID;
		}

		if ( is_wp_error( $user_id ) ) {
			$response = array(
				'status' => 'error',
				'error'  => $user_id->get_error_message(),
			);
		} else {
			$response = array(
				'status'  => 'success',
				'user_id' => $user_id,
			);

			if ( is_array( $user_roles ) ) {
				foreach ( $user_roles as $user_role ) {
					$this->add_user_role(
						array(
							'user_role' => trim( $user_role ),
							'user_id'   => $user_id,
						)
					);
				}
			}

			if ( $notify ) {
				wp_new_user_notification( $user_id, null, $notify );
			}
		}

		return wp_json_encode( $response );
	}

	/**
	 * Updayte an existing user.
	 *
	 * @access public
	 * @since 1.0
	 * @param array $data Request data.
	 * @return array
	 */
	public function update_existing_user( $data ) {
		$response = array();

		$user_roles = isset( $data['user_role'] ) ? explode( ',', $data['user_role'] ) : '';

		$user_data = array(
			'ID'         => $data['user_id'], // User ID of the user to update.
			'user_email' => $data['email'],
			'first_name' => isset( $data['first_name'] ) ? $data['first_name'] : '',
			'last_name'  => isset( $data['last_name'] ) ? $data['last_name'] : '',
			'role'       => is_array( $user_roles ) ? $user_roles[0] : $user_roles,
		);

		$user = get_user_by( 'ID', $data['user_id'] );

		if ( $user ) {
			$user_id = wp_update_user( $user_data ); // Update the existing user.
		} else {
			$response = array(
				'status' => 'error',
				'error'  => 'User not found.', // User doesn't exist, so cannot update.
			);
			return wp_json_encode( $response );
		}

		if ( is_wp_error( $user_id ) ) {
			$response = array(
				'status' => 'error',
				'error'  => $user_id->get_error_message(),
			);
		} else {
			$response = array(
				'status'  => 'success',
				'user_id' => $user_id,
			);

			if ( is_array( $user_roles ) ) {
				foreach ( $user_roles as $user_role ) {
					$this->add_user_role(
						array(
							'user_role' => trim( $user_role ),
							'user_id'   => $user_id,
						)
					);
				}
			}
		}

		return wp_json_encode( $response );
	}

	/**
	 * Create new comment.
	 *
	 * @access public
	 * @since 1.0
	 * @param array $data Request data.
	 * @return array
	 */
	public function create_new_comment( $data ) {
		$response = array();

		// Create comment object.
		$comment_data = array(
			'comment_post_ID'      => $data['post_id'],
			'comment_content'      => $data['comment'],
			'comment_author'       => $data['author'],
			'comment_author_email' => $data['email'],
			'comment_status'       => 'approved', // Update the status field to approved.
			'comment_author_url'   => $data['author_url'], // The url of the author.
			'comment_type'         => '', // The type of comment (if any).
			'comment_parent'       => 0, // The parent comment (if any).
			'comment_author_IP'    => '', // The IP address of the author.
			'comment_agent'        => 'FlowMattic/' . FLOWMATTIC_VERSION, // The agent used by the author.
			'comment_date'         => date( 'Y-m-d H:i:s' ), // The date of the comment.
			'comment_approved'     => 1, // If the comment needs to be approved by an admin.
		);

		// Insert the comment into the database.
		$comment_id = wp_new_comment( $comment_data, true );

		// Error response.
		if ( is_wp_error( $comment_id ) ) {
			$response = array(
				'status' => 'error',
				'error'  => $comment_id->get_error_message(),
			);
		} else {
			$response = array(
				'status'     => 'success',
				'comment_id' => $comment_id,
			);
		}

		return wp_json_encode( $response );
	}

	/**
	 * Create new post.
	 *
	 * @access public
	 * @since 1.0
	 * @param array $data Request data.
	 * @return array
	 */
	public function create_new_post( $data ) {
		$response = array();

		// Create post object.
		$post_args = array(
			'post_title'   => wp_strip_all_tags( $data['post_title'] ),
			'post_content' => $data['post_content'],
			'post_status'  => $data['post_status'],
			'post_type'    => $data['post_type'],
			'meta_input'   => isset( $data['custom_fields'] ) ? $data['custom_fields'] : array(),
		);

		// If categories are set, insert them.
		if ( isset( $data['post_category'] ) && ! empty( $data['post_category'] ) ) {
			$post_args['post_category'] = $data['post_category'];
		}

		// If tags are set, insert them.
		if ( isset( $data['post_tags'] ) && ! empty( $data['post_tags'] ) ) {
			$post_args['tags_input'] = $data['post_tags'];
		}

		// If author id is set, update them.
		if ( isset( $data['post_author_id'] ) && ! empty( $data['post_author_id'] ) ) {
			$user_id = $data['post_author_id'];

			if ( is_email( $user_id ) ) {
				$user    = get_user_by( 'email', $fields['user_email'] );
				$user_id = $user ? $user->ID : 1;
			}

			$post_args['post_author'] = $user_id;
		}

		// If post excerpt is available, update.
		if ( ! empty( $data['post_excerpt'] ) ) {
			$post_args['post_excerpt'] = $data['post_excerpt'];
		}

		// Insert the post into the database.
		$post_id = wp_insert_post( $post_args, true );

		// Check if post thumbnail id is provided.
		$post_thumbnail_id = '';
		if ( isset( $data['post_thumbnail_id'] ) && '' !== $data['post_thumbnail_id'] ) {

			// Get the image ID.
			$post_thumbnail_id = $data['post_thumbnail_id'];

			// If image URL provided, import it first, and then use the image ID.
			if ( ! is_numeric( $post_thumbnail_id ) ) {
				$import_image      = $this->create_new_media( array( 'file' => $post_thumbnail_id ) );
				$import_image      = json_decode( $import_image, true );
				$post_thumbnail_id = isset( $import_image['media_id'] ) ? $import_image['media_id'] : '';
			}

			// Set the post featured image.
			set_post_thumbnail( $post_id, $post_thumbnail_id );
		}

		if ( is_wp_error( $post_id ) ) {
			$response = array(
				'status' => 'error',
				'error'  => $post_id->get_error_message(),
			);
		} else {
			$response = array(
				'status'  => 'success',
				'post_id' => $post_id,
			);
		}

		return wp_json_encode( $response );
	}

	/**
	 * Update post.
	 *
	 * @access public
	 * @since 1.2
	 * @param array $data Request data.
	 * @return array
	 */
	public function update_post( $data ) {
		$response = array();

		// Create post object.
		$post_args = array(
			'ID' => $data['post_id'],
		);

		// If post title is available, update.
		if ( ! empty( $data['post_title'] ) ) {
			$post_args['post_title'] = wp_strip_all_tags( $data['post_title'] );
		}

		// If post content is available, update.
		if ( ! empty( $data['post_content'] ) ) {
			$post_args['post_content'] = $data['post_content'];
		}

		// If post status is available, update.
		if ( ! empty( $data['post_status'] ) ) {
			$post_args['post_status'] = $data['post_status'];
		}

		// If custom fields is available, update.
		if ( ! empty( $data['custom_fields'] ) ) {
			$post_args['meta_input'] = isset( $data['custom_fields'] ) ? $data['custom_fields'] : array();
		}

		// If post excerpt is available, update.
		if ( ! empty( $data['post_excerpt'] ) ) {
			$post_args['post_excerpt'] = $data['post_excerpt'];
		}

		// If post type is set, update it..
		if ( isset( $data['post_type'] ) && ! empty( $data['post_type'] ) ) {
			$post_args['post_type'] = $data['post_type'];
		}

		// If categories are set, update them.
		if ( isset( $data['post_category'] ) && ! empty( $data['post_category'] ) ) {
			$post_args['post_category'] = $data['post_category'];
		}

		// If tags are set, update them.
		if ( isset( $data['post_tags'] ) && ! empty( $data['post_tags'] ) ) {
			$post_args['tags_input'] = $data['post_tags'];
		}

		// If author id is set, update them.
		if ( isset( $data['post_author_id'] ) && ! empty( $data['post_author_id'] ) ) {
			$user_id = $data['post_author_id'];

			if ( is_email( $user_id ) ) {
				$user    = get_user_by( 'email', $fields['user_email'] );
				$user_id = $user->ID;
			}

			$post_args['post_author'] = $user_id;
		}

		// Update the post into the database.
		$post_id = wp_update_post( $post_args, true );

		// Check if post thumbnail id is provided.
		$post_thumbnail_id = '';
		if ( isset( $data['post_thumbnail_id'] ) && '' !== $data['post_thumbnail_id'] ) {

			// Get the image ID.
			$post_thumbnail_id = $data['post_thumbnail_id'];

			// If image URL provided, import it first, and then use the image ID.
			if ( ! is_numeric( $post_thumbnail_id ) ) {
				$import_image      = $this->create_new_media( array( 'file' => $post_thumbnail_id ) );
				$import_image      = json_decode( $import_image, true );
				$post_thumbnail_id = isset( $import_image['media_id'] ) ? $import_image['media_id'] : '';
			}

			// Set the post featured image.
			set_post_thumbnail( $post_id, $post_thumbnail_id );
		}

		if ( is_wp_error( $post_id ) ) {
			$response = array(
				'status' => 'error',
				'error'  => $post_id->get_error_message(),
			);
		} else {
			$response = array(
				'status'  => 'success',
				'post_id' => $post_id,
			);
		}

		return wp_json_encode( $response );
	}

	/**
	 * Create new media.
	 *
	 * @access public
	 * @since 1.0
	 * @param array $data Request data.
	 * @return array
	 */
	public function create_new_media( $data ) {
		$response = array();
		$file     = $data['file'];

		add_filter( 'https_local_ssl_verify', '__return_false' );
		add_filter( 'https_ssl_verify', '__return_false' );

		// Determine the file type based on the URL.
		$filetype = wp_check_filetype( $file );

		// Manually set the file extension if the URL doesn't have one.
		if ( empty( $filetype['ext'] ) ) {
			$filetype['ext'] = 'jpg'; // Manually set the extension to match the image type.
		}

		// Construct a new filename based on the URL and file extension.
		$filename = basename( $file );

		$file_array = array(
			'name'     => $filename,
			'tmp_name' => flowmattic_download_url( $file, 300, false ),
		);

		if ( is_wp_error( $file_array['tmp_name'] ) ) {
			$response = array(
				'status' => 'error',
				'error'  => $file_array['tmp_name']->get_error_message(),
			);
		} else {
			$upload_overrides = array(
				'test_form' => false,
				'test_size' => false,
				'test_type' => false,
				'action'    => 'flowmattic_handle_upload',
			);

			if ( ! function_exists( 'media_handle_sideload' ) ) {
				require_once ABSPATH . 'wp-admin/includes/media.php';
				require_once ABSPATH . 'wp-admin/includes/file.php';
				require_once ABSPATH . 'wp-admin/includes/image.php';
			}

			$media = media_handle_sideload( $file_array, 0, null, $upload_overrides );

			if ( is_wp_error( $media ) ) {
				$response = array(
					'status' => 'error',
					'error'  => $media->get_error_message(),
				);
			} else {
				$response = array(
					'status'    => 'success',
					'media_id'  => $media,
					'media_url' => wp_get_attachment_url( $media ),
				);
			}
		}

		return wp_json_encode( $response );
	}

	/**
	 * Update user meta.
	 *
	 * @access public
	 * @since 1.0
	 * @param array $data Request data.
	 * @return array
	 */
	public function update_user_meta( $data ) {
		$response = array();

		$meta_fields = $data['meta_fields'];
		$user_id     = $data['user_id'];

		foreach ( $meta_fields as $key => $value ) {
			$status = update_user_meta( $user_id, $key, $value );

			if ( $status ) {
				$response[ $key . ' - ID' ] = $status;
			} else {
				$response[ $key ] = esc_html__( 'No values updated.', 'flowmattic' );
			}
		}

		return wp_json_encode( $response );
	}

	/**
	 * Get user profile by ID.
	 *
	 * @access public
	 * @since 1.2
	 * @param array $fields User data fields.
	 * @return array
	 */
	public function get_user_by_id( $fields ) {
		$user = get_user_by( 'id', $fields['user_id'] );

		if ( $user ) {
			$post_data = (array) $user->data;
			$user_meta = $this->get_user_metadata( $fields['user_id'] );
			$post_data = array_merge( $post_data, $user_meta );

			// Remove the user passwords from the received data for privacy concern.
			unset( $post_data['password'] );
			unset( $post_data['user_pass'] );
			unset( $post_data['userpass'] );
			unset( $post_data['user_password'] );

			return wp_json_encode( $post_data );
		} else {
			return wp_json_encode(
				array(
					'status'  => 'error',
					'message' => esc_html__( 'User not found for the given ID', 'flowmattic' ),
					'user_id' => $fields['user_id'],
				)
			);
		}
	}

	/**
	 * Get new user profile by email.
	 *
	 * @access public
	 * @since 1.2
	 * @param array $fields User data fields.
	 * @return array
	 */
	public function get_user_by_email( $fields ) {
		$user = get_user_by( 'email', $fields['user_email'] );

		if ( $user ) {
			$post_data = (array) $user->data;
			$user_meta = $this->get_user_metadata( $user->ID );
			$post_data = array_merge( $post_data, $user_meta );

			// Remove the user passwords from the received data for privacy concern.
			unset( $post_data['password'] );
			unset( $post_data['user_pass'] );
			unset( $post_data['userpass'] );
			unset( $post_data['user_password'] );

			return wp_json_encode( $post_data );
		} else {
			return wp_json_encode(
				array(
					'status'     => 'error',
					'message'    => esc_html__( 'User not found for the given email', 'flowmattic' ),
					'user_email' => $fields['user_email'],
				)
			);
		}
	}

	/**
	 * Create category.
	 *
	 * @access public
	 * @since 1.2
	 * @param array $data Request data.
	 * @return array
	 */
	public function create_category( $data ) {
		$response = array();

		$category_name = $data['category_name'];

		// Create the main category.
		$category = wp_insert_term(
			// The name of the category.
			$category_name,
			// The taxonomy, which in this case if category (don't change).
			'category',
			array()
		);

		if ( is_wp_error( $category ) ) {
			$response = array(
				'status' => 'error',
				'error'  => $category->get_error_message(),
			);
		} else {
			$response = $category;
		}

		return wp_json_encode( $response );
	}

	/**
	 * Create tag.
	 *
	 * @access public
	 * @since 1.2
	 * @param array $data Request data.
	 * @return array
	 */
	public function create_tag( $data ) {
		$response = array();

		$tag_name = $data['tag_name'];

		// Create the main tag.
		$tag = wp_insert_term(
			// The name of the tag.
			$tag_name,
			// The taxonomy, which in this case if tag (don't change).
			'post_tag',
			array()
		);

		if ( is_wp_error( $tag ) ) {
			$response = array(
				'status' => 'error',
				'error'  => $tag->get_error_message(),
			);
		} else {
			$response = $tag;
		}

		return wp_json_encode( $response );
	}

	/**
	 * Create role.
	 *
	 * @access public
	 * @since 2.1.0
	 * @param array $data Request data.
	 * @return array
	 */
	public function add_role( $data ) {
		$response = array();

		$role_name         = $data['role_name'];
		$role_display_name = $data['role_display_name'];
		$role_capabilities = ( isset( $data['role_capabilities'] ) && '' !== $data['role_capabilities'] ) ? explode( ',', $data['role_capabilities'] ) : array();

		// Add role to WordPress.
		$new_role = add_role( $role_name, $role_display_name, $role_capabilities );

		if ( ! $new_role ) {
			$response = array(
				'status' => 'error',
				'error'  => esc_html__( 'Looks like user role already exists', 'flowmattic' ),
			);
		} else {
			$response = $new_role;
		}

		return wp_json_encode( $response );
	}

	/**
	 * Add role to existing user.
	 *
	 * @access public
	 * @since 1.2
	 * @param array $data Request data.
	 * @return array
	 */
	public function add_user_role( $data ) {
		$response = array();

		if ( ! isset( $data['user_role'] ) ) {
			return wp_json_encode(
				array(
					'status'  => 'error',
					'message' => esc_html__( 'User role is not provided', 'flowmattic' ),
				)
			);
		}

		if ( ! isset( $data['user_id'] ) ) {
			return wp_json_encode(
				array(
					'status'  => 'error',
					'message' => esc_html__( 'User ID is not provided', 'flowmattic' ),
				)
			);
		}

		$user_role = $data['user_role'];
		$user_id   = $data['user_id'];

		// Instantiate the WP User.
		$user = new WP_User( $user_id );

		// Add the new user role to the user.
		$user_roles = isset( $data['user_role'] ) ? explode( ',', $data['user_role'] ) : '';

		if ( is_array( $user_roles ) ) {
			foreach ( $user_roles as $key => $role ) {
				$user->add_role( $role );
			}
		} else {
			$user->add_role( $user_role );
		}

		return wp_json_encode(
			array(
				'status'  => 'status',
				'message' => esc_html__( 'User role is successfully added to the user', 'flowmattic' ),
			)
		);
	}

	/**
	 * Change user role to existing user.
	 *
	 * @access public
	 * @since 1.2
	 * @param array $data Request data.
	 * @return array
	 */
	public function change_user_role( $data ) {
		$response = array();

		if ( ! isset( $data['user_role'] ) ) {
			return wp_json_encode(
				array(
					'status'  => 'error',
					'message' => esc_html__( 'User role is not provided', 'flowmattic' ),
				)
			);
		}

		if ( ! isset( $data['user_id'] ) ) {
			return wp_json_encode(
				array(
					'status'  => 'error',
					'message' => esc_html__( 'User ID is not provided', 'flowmattic' ),
				)
			);
		}

		$user_role = $data['user_role'];
		$user_id   = $data['user_id'];

		// Instantiate the WP User.
		$user = new WP_User( $user_id );

		// Change the user role from the user.
		$user_roles = isset( $data['user_role'] ) ? explode( ',', $data['user_role'] ) : '';

		if ( is_array( $user_roles ) ) {
			foreach ( $user_roles as $key => $role ) {
				$user->set_role( $role );
			}
		} else {
			$user->set_role( $user_role );
		}

		return wp_json_encode(
			array(
				'status'  => 'status',
				'message' => esc_html__( 'User role is successfully changed', 'flowmattic' ),
			)
		);
	}

	/**
	 * Remove user role to existing user.
	 *
	 * @access public
	 * @since 1.2
	 * @param array $data Request data.
	 * @return array
	 */
	public function remove_user_role( $data ) {
		$response = array();

		if ( ! isset( $data['user_role'] ) ) {
			return wp_json_encode(
				array(
					'status'  => 'error',
					'message' => esc_html__( 'User role is not provided', 'flowmattic' ),
				)
			);
		}

		if ( ! isset( $data['user_id'] ) ) {
			return wp_json_encode(
				array(
					'status'  => 'error',
					'message' => esc_html__( 'User ID is not provided', 'flowmattic' ),
				)
			);
		}

		$user_role = $data['user_role'];
		$user_id   = $data['user_id'];

		// Instantiate the WP User.
		$user = new WP_User( $user_id );

		// Remove the user role from the user.
		$user_roles = isset( $data['user_role'] ) ? explode( ',', $data['user_role'] ) : '';

		if ( is_array( $user_roles ) ) {
			foreach ( $user_roles as $key => $role ) {
				$user->remove_role( $role );
			}
		} else {
			$user->remove_role( $user_role );
		}

		return wp_json_encode(
			array(
				'status'     => 'status',
				'message'    => esc_html__( 'User role is successfully removed', 'flowmattic' ),
				'id'         => $user_id,
				'user_roles' => $user_roles,
			)
		);
	}

	/**
	 * Delete user account.
	 *
	 * @access public
	 * @since 1.2
	 * @param array $data Request data.
	 * @return array
	 */
	public function delete_user( $data ) {
		$response = array();

		if ( ! isset( $data['user_id'] ) ) {
			return wp_json_encode(
				array(
					'status'  => 'error',
					'message' => esc_html__( 'User ID is not provided', 'flowmattic' ),
				)
			);
		}

		$user_id     = $data['user_id'];
		$reassign_id = isset( $data['reassign_id'] ) ? $data['reassign_id'] : null;

		// Make sure wp_delete_user function exists.
		require_once ABSPATH . 'wp-admin/includes/user.php';

		// Delete the WP User.
		wp_delete_user( $user_id, $reassign_id );

		return wp_json_encode(
			array(
				'status'  => 'status',
				'message' => esc_html__( 'User account successfully deleted', 'flowmattic' ),
			)
		);
	}

	/**
	 * Trigger: User login.
	 *
	 * @access public
	 * @since 1.2
	 * @param string $user_login Username of the logged in user.
	 * @param object $user       WP User object of the logged in user.
	 * @return void
	 */
	public function capture_user_login( $user_login, $user ) {
		$workflow_id = $this->is_capturing();

		$user_data = (array) $user->data;

		// Remove the user passwords from the received data for privacy concern.
		unset( $user_data['password'] );
		unset( $user_data['user_pass'] );
		unset( $user_data['userpass'] );
		unset( $user_data['user_password'] );

		// If workflow ID is set, the webhook capture event is in progress.
		if ( $workflow_id ) {
			$trigger_event = get_option( 'webhook-capture-app-action', false );
			if ( $trigger_event && 'user_login' === $trigger_event ) {
				update_option( 'webhook-capture-' . $workflow_id, $user_data, false );
			}
		}

		// Run the workflow.
		$this->run_workflow_step( 'user_login', $user_data );
	}

	/**
	 * Get post metadata.
	 *
	 * @access public
	 * @since 2.2.0
	 * @param array $fields Workflow captured data.
	 * @return array
	 */
	public function get_post_meta( $fields ) {
		$post_id       = (int) $fields['post_id'];
		$post_meta_key = isset( $fields['post_meta_key'] ) ? $fields['post_meta_key'] : '';

		$simple_entry = array();

		// If no Post ID, return default message.
		if ( ! $post_id ) {
			return wp_json_encode(
				array(
					'status'  => 'error',
					'message' => esc_html__( 'Post ID required. Check incoming data or dynamic tag values.', 'flowmattic' ),
				)
			);
		}

		// If single post meta.
		if ( '' !== $post_meta_key ) {

			// If ACF is installed, try getting the data from ACF fields.
			if ( function_exists( 'get_field' ) ) {
				$post_meta_data = get_field( $post_meta_key, $post_id );

				$simple_entry = array(
					'post_meta' => $post_meta_key,
				);

				if ( $post_meta_data ) {
					if ( is_array( $post_meta_data ) ) {
						$array_value = wp_json_encode( $post_meta_data );

						$simple_entry['acf_array_value'] = $array_value;
					} else {
						$simple_entry['value'] = $post_meta_data;
					}
				} else {
					$post_meta_data = get_post_meta( $post_id, $post_meta_key, true );

					$simple_entry['value'] = $post_meta_data;
				}
			} else {
				$post_meta_data = get_post_meta( $post_id, $post_meta_key, true );

				$simple_entry = array(
					'post_meta' => $post_meta_key,
					'value'     => $post_meta_data,
				);
			}
		} else {
			$post_meta_data = get_post_meta( $post_id );
			$post_meta_data = is_array( $post_meta_data ) ? array_combine( array_keys( $post_meta_data ), array_column( $post_meta_data, '0' ) ) : array();

			foreach ( $post_meta_data as $key => $post_meta ) {
				// If ACF is installed, try getting the data from ACF fields.
				if ( function_exists( 'get_field' ) ) {
					$post_meta = get_field( $key, $post_id );

					if ( is_array( $post_meta ) ) {
						$post_meta = wp_json_encode( $post_meta );
					}
				}
				$simple_entry[ $key ] = $post_meta;
			}
		}

		// get the post object by ID.
		$post = get_post( $post_id );

		$post_title     = $post->post_title;  // get the post title.
		$post_permalink = get_permalink( $post_id );  // get the post permalink.
		$post_type      = $post->post_type;  // get the post type.
		$post_author_id = $post->post_author;  // get the author ID.
		$post_author    = get_the_author_meta( 'display_name', $post_author_id );  // get the author name.

		if ( '' === $post_meta_key ) {
			$simple_entry['post_title']     = $post_title;
			$simple_entry['post_permalink'] = $post_permalink;
			$simple_entry['post_type']      = $post_type;
			$simple_entry['post_author_id'] = $post_author_id;
			$simple_entry['post_author']    = $post_author;
		}

		return wp_json_encode( $simple_entry );
	}

	/**
	 * Get Posts by post_type query
	 *
	 * @since 2.2.2
	 * @param array $fields Workflow captured data.
	 * @return string JSON String.
	 */
	public function get_posts_by_post_type( $fields ) {
		$app_args        = ( isset( $fields['actionAppArgs'] ) ) ? $fields['actionAppArgs'] : $fields['settings']['actionAppArgs'];
		$post_type       = $app_args['post_type'];
		$posts_per_page  = $app_args['posts_per_page'];
		$order           = $app_args['order'];
		$orderby         = $app_args['orderby'];
		$single_array    = ( isset( $app_args['all_posts_as_array'] ) && 'No' === $app_args['all_posts_as_array'] ) ? false : true;
		$exclude_content = ( isset( $app_args['exclude_post_content'] ) && 'Yes' === $app_args['exclude_post_content'] ) ? true : false;
		$exclude_meta    = ( isset( $app_args['exclude_post_meta'] ) && 'Yes' === $app_args['exclude_post_meta'] ) ? true : false;

		// Prepare query data.
		$args = array(
			'post_type' => $post_type,
		);

		if ( '' !== $posts_per_page ) {
			$args['posts_per_page'] = $posts_per_page;
		}
		if ( '' !== $order ) {
			$args['order'] = $order;
		}
		if ( '' !== $orderby ) {
			$args['orderby'] = $orderby;
		}

		// Run query.
		$posts = get_posts( $args );

		// Check results.
		if ( ! $posts || is_wp_error( $posts ) ) {
			return false;
		}

		foreach ( $posts as $key => $post_object ) {
			$post_object = (array) $post_object;

			if ( $exclude_content ) {
				unset( $post_object['post_content'] );
			} else {
				// Convert double quotes in post content to html entity.
				$post_object['post_content'] = esc_attr( $post_object['post_content'] );
			}

			// Convert double quotes in post title to html entity.
			$post_object['post_title'] = esc_attr( $post_object['post_title'] );

			if ( ! $single_array ) {
				$posts[ $key ]    = array();
				$posts[ $key ][0] = $post_object;
			} else {
				$posts[ $key ] = $post_object;
			}

			$post_id        = $post_object['ID'];
			$post_meta_data = array();

			if ( $exclude_meta ) {
				unset( $posts[ $key ]['post_meta'] );
			} else {
				// Get all meta.
				$post_meta_data = get_post_meta( $post_id );
				$post_meta_data = array_combine( array_keys( $post_meta_data ), array_column( $post_meta_data, '0' ) );

				foreach ( $post_meta_data as $metakey => $post_meta ) {
					// If ACF is installed, try getting the data from ACF fields.
					if ( function_exists( 'get_field' ) ) {
						$post_meta = get_field( $metakey, $post_id );
					}

					$post_meta_array = array();
					if ( is_array( $post_meta ) ) {
						foreach ( $post_meta as $meta_key => $meta ) {
							$post_meta_array[ $key ] = is_array( $meta ) ? array_map( 'addslashes', $meta ) : addslashes( $meta );
						}
						$post_meta = $post_meta_array;
					} else {
						$post_meta = addslashes( $post_meta );
					}

					$post_meta_data[ $metakey ] = $post_meta;
				}

				if ( ! $single_array ) {
					$posts[ $key ][0]['post_meta'] = $post_meta_data;
				} else {
					$posts[ $key ]['post_meta'] = $post_meta_data;
				}
			}
		}

		if ( $single_array ) {
			// Prepare for iterator.
			$posts = array(
				'all_posts' => stripslashes( wp_json_encode( $posts ) ),
			);
		} else {
			$posts = array_map( 'wp_json_encode', $posts );
		}

		// Send back results.
		return wp_json_encode( $posts );
	}

	/**
	 * Get Posts by post_meta query
	 *
	 * @since 2.2.2
	 * @param array $fields Workflow captured data.
	 * @return string JSON String.
	 */
	public function get_posts_by_meta( $fields ) {
		$app_args        = ( isset( $fields['actionAppArgs'] ) ) ? $fields['actionAppArgs'] : $fields['settings']['actionAppArgs'];
		$post_type       = $app_args['post_type'];
		$meta_key        = $app_args['post_meta_key'];
		$meta_value      = $app_args['post_meta_value'];
		$single_array    = ( isset( $app_args['all_posts_as_array'] ) && 'No' === $app_args['all_posts_as_array'] ) ? false : true;
		$exclude_content = ( isset( $app_args['exclude_post_content'] ) && 'Yes' === $app_args['exclude_post_content'] ) ? true : false;
		$exclude_meta    = ( isset( $app_args['exclude_post_meta'] ) && 'Yes' === $app_args['exclude_post_meta'] ) ? true : false;

		// Prepare query data.
		$args = array(
			'meta_query'     => array( // @codingStandardsIgnoreLine
				array(
					'key'   => $meta_key,
					'value' => $meta_value,
				),
			),
			'post_type'      => $post_type,
			'posts_per_page' => '-1',
		);

		// Run query.
		$posts = get_posts( $args );

		// Check results.
		if ( ! $posts || is_wp_error( $posts ) ) {
			return false;
		}

		foreach ( $posts as $key => $post_object ) {
			$post_object    = (array) $post_object;
			$posts[ $key ]  = $post_object;
			$post_id        = $post_object['ID'];
			$post_meta_data = array();

			if ( $exclude_content ) {
				unset( $posts[ $key ]['post_content'] );
			} else {
				// Convert double quotes in post content to html entity.
				$posts[ $key ]['post_content'] = esc_attr( $posts[ $key ]['post_content'] );
			}

			// Convert double quotes in post title to html entity.
			$post_object['post_title'] = esc_attr( $post_object['post_title'] );

			if ( $exclude_meta ) {
				unset( $posts[ $key ]['post_meta'] );
			} else {
				// Get all meta.
				$post_meta_data = get_post_meta( $post_id );
				$post_meta_data = array_combine( array_keys( $post_meta_data ), array_column( $post_meta_data, '0' ) );

				foreach ( $post_meta_data as $metakey => $post_meta ) {
					// If ACF is installed, try getting the data from ACF fields.
					if ( function_exists( 'get_field' ) ) {
						$post_meta = get_field( $metakey, $post_id );
					}

					$post_meta_data[ $metakey ] = $post_meta;
				}

				$posts[ $key ]['post_meta'] = $post_meta_data;
			}
		}

		if ( $single_array ) {
			// Prepare for iterator.
			$posts = array(
				'all_posts' => stripslashes( wp_json_encode( $posts ) ),
			);
		} else {
			$posts = array_map( 'wp_json_encode', $posts );
		}

		// Send back results.
		return wp_json_encode( $posts );
	}

	/**
	 * Get user metadata.
	 *
	 * @access public
	 * @since 2.2.0
	 * @param array $fields Workflow captured data.
	 * @return array
	 */
	public function get_user_meta( $fields ) {
		$user_id       = (int) $fields['user_id'];
		$user_meta_key = isset( $fields['user_meta_key'] ) ? $fields['user_meta_key'] : '';

		$simple_entry = array();

		// If single user meta.
		if ( '' !== $user_meta_key ) {

			// If ACF is installed, try getting the data from ACF fields.
			if ( function_exists( 'get_field' ) ) {
				$user_meta_data = get_field( $user_meta_key, 'user_' . $user_id );

				$simple_entry = array(
					'user_meta' => $user_meta_key,
				);

				if ( $user_meta_data ) {
					if ( is_array( $user_meta_data ) ) {
						$array_value = wp_json_encode( $user_meta_data );

						$simple_entry['acf_array_value'] = $array_value;
					} else {
						$simple_entry['value'] = $user_meta_data;
					}
				} else {
					$user_meta_data = get_user_meta( $user_id, $user_meta_key, true );

					$simple_entry['value'] = $user_meta_data;
				}
			} else {
				$user_meta_data = get_user_meta( $user_id, $user_meta_key, true );

				$simple_entry = array(
					'user_meta' => $user_meta_key,
					'value'     => $user_meta_data,
				);
			}
		} else {
			$simple_entry = $this->get_user_metadata( $user_id );
		}

		return wp_json_encode( $simple_entry );
	}

	/**
	 * Get WP user metadata.
	 *
	 * @access public
	 * @since 2.2.0
	 * @param int $user_id User ID.
	 * @return array
	 */
	public function get_user_metadata( $user_id ) {
		$simple_entry   = array();
		$user_meta_data = get_user_meta( $user_id );

		foreach ( $user_meta_data as $key => $user_meta ) {
			// If ACF is installed, try getting the data from ACF fields.
			if ( function_exists( 'get_field' ) ) {
				$user_meta = get_field( $key, 'user_' . $user_id );
			} else {
				$user_meta = maybe_unserialize( $user_meta[0] );
			}

			if ( is_array( $user_meta ) ) {
				$user_meta = wp_json_encode( $user_meta );
			}

			$simple_entry[ $key ] = $user_meta;
		}

		return $simple_entry;
	}

	/**
	 * Get all WP user roles.
	 *
	 * @access public
	 * @since 2.0
	 * @return array
	 */
	public function get_all_user_roles() {
		$all_user_roles = array();
		$user_roles     = wp_roles();
		foreach ( $user_roles->roles as $name => $role ) {
			$all_user_roles[ $name ] = esc_attr( $role['name'] );
		}

		return $all_user_roles;
	}

	/**
	 * Get all WP user roles capabilities.
	 *
	 * @access public
	 * @since 2.0
	 * @return array
	 */
	public function get_all_user_role_capabilities() {
		$all_user_role_capabilities = array();
		$user_roles                 = wp_roles();

		foreach ( $user_roles->roles as $name => $role ) {
			foreach ( $role['capabilities'] as $key => $cap ) {
				$all_user_role_capabilities[ $key ] = esc_attr( $key );
			}
		}

		return $all_user_role_capabilities;
	}

	/**
	 * Test action event ajax.
	 *
	 * @access public
	 * @since 1.0
	 * @param array $event_data Test event data.
	 * @return array
	 */
	public function test_event_action( $event_data ) {
		// Set true to avoid running the entire workflow.
		$this->is_doing_test = true;

		$response    = array();
		$event       = $event_data['event'];
		$fields      = $event_data['settings'];
		$workflow_id = $event_data['workflow_id'];
		$step_ids    = $event_data['stepIDs'];

		if ( isset( $event_data['fields']['user_id'] ) ) {
			$fields['user_id'] = $event_data['fields']['user_id'];
		}

		if ( isset( $event_data['fields']['user_email'] ) ) {
			$fields['user_email'] = $event_data['fields']['user_email'];
		}

		switch ( $event ) {
			case 'new_user':
				$response = $this->create_new_user( $fields );
				break;

			case 'update_user':
				$response = $this->update_existing_user( $fields );
				break;

			case 'new_comment':
				$response = $this->create_new_comment( $fields );
				break;

			case 'new_post':
				$response = $this->create_new_post( $fields );
				break;

			case 'update_post':
				$response = $this->update_post( $fields );
				break;

			case 'new_media':
				$response = $this->create_new_media( $fields );
				break;

			case 'update_user_meta':
				$response = $this->update_user_meta( $fields );
				break;

			case 'get_user_by_id':
				$response = $this->get_user_by_id( $fields );
				break;

			case 'get_user_by_email':
				$response = $this->get_user_by_email( $fields );
				break;

			case 'create_category':
				if ( isset( $event_data['fields']['category_name'] ) ) {
					$fields['category_name'] = $event_data['fields']['category_name'];
				}

				$response = $this->create_category( $fields );
				break;

			case 'create_tag':
				if ( isset( $event_data['fields']['tag_name'] ) ) {
					$fields['tag_name'] = $event_data['fields']['tag_name'];
				}

				$response = $this->create_tag( $fields );
				break;

			case 'add_user_role':
				$response = $this->add_user_role( $fields );
				break;

			case 'change_user_role':
				$response = $this->change_user_role( $fields );
				break;

			case 'remove_user_role':
				$response = $this->remove_user_role( $fields );
				break;

			case 'delete_user':
				$response = $this->delete_user( $fields );
				break;

			case 'add_role':
				if ( isset( $event_data['fields'] ) ) {
					$fields = $event_data['fields'];
				}

				$response = $this->add_role( $fields );
				break;

			case 'get_post_meta':
				$response = $this->get_post_meta( $fields );
				break;

			case 'get_user_meta':
				$response = $this->get_user_meta( $fields );
				break;

			case 'get_posts_by_meta':
				$response = $this->get_posts_by_meta( $fields );
				break;

			case 'get_posts_by_post_type':
				$response = $this->get_posts_by_post_type( $fields );
				break;
		}

		return $response;
	}
}

new FlowMattic_Wordpress();
