<?php
/**
 * Handle database queries for connects.
 *
 * @package flowmattic
 * @since 3.0
 */

if ( ! class_exists( 'FlowMattic_Database_Connects' ) ) {
	/**
	 * Handle database queries for connects.
	 *
	 * @since 3.0
	 */
	class FlowMattic_Database_Connects {

		/**
		 * The table name.
		 *
		 * @access protected
		 * @since 3.0
		 * @var string
		 */
		protected $table_name = 'flowmattic_connects';

		/**
		 * The Constructor.
		 *
		 * @since 3.0
		 * @access public
		 */
		public function __construct() {
		}

		/**
		 * Insert connect to database.
		 *
		 * @since 3.0
		 * @access public
		 * @param array $args The arguments.
		 * @return integer|boolean The last insert id or false if query failed.
		 */
		public function insert( $args ) {
			global $wpdb;

			// Check if connect ID is not provided, else update.
			if ( ! isset( $args['connect_id'] ) || '' === trim( $args['connect_id'] ) ) {
				$status = $wpdb->insert(
					$wpdb->prefix . $this->table_name,
					array(
						'connect_id'       => flowmattic_random_string( 8 ),
						'connect_name'     => esc_attr( $args['connect_name'] ),
						'connect_data'     => '',
						'connect_settings' => base64_encode( wp_json_encode( $args['connect_settings'] ) ),
						'connect_time'     => date_i18n( 'Y-m-d H:i:s' ),
					)
				);

				return $wpdb->insert_id;
			} else {
				$exists = $this->get( $args );

				// If not exists, add new record, else update.
				if ( $exists ) {
					return $this->update( $args );
				}
			}
		}

		/**
		 * Update the connect in database.
		 *
		 * @since 3.0
		 * @access public
		 * @param array $args The arguments.
		 * @return void
		 */
		public function update( $args ) {
			global $wpdb;

			// Check if connect ID is provided, else skip.
			if ( isset( $args['connect_id'] ) ) {
				$connect_id = (int) $args['connect_id'];

				$data_args = array();

				if ( isset( $args['connect_name'] ) && '' !== $args['connect_name'] ) {
					$data_args['connect_name'] = $args['connect_name'];
				}

				if ( isset( $args['connect_data'] ) && '' !== $args['connect_data'] ) {
					$data_args['connect_data'] = $args['connect_data'];
				}

				if ( isset( $args['connect_settings'] ) && '' !== $args['connect_settings'] ) {
					$data_args['connect_settings'] = $args['connect_settings'];
				}

				// Update the entry to database.
				$wpdb->update(
					$wpdb->prefix . $this->table_name,
					$data_args,
					array(
						'id' => esc_attr( $connect_id ),
					)
				);
			}
		}

		/**
		 * Delete the connect from database.
		 *
		 * @since 3.0
		 * @access public
		 * @param array $args The arguments.
		 * @return void
		 */
		public function delete( $args ) {
			global $wpdb;

			// Check if connect ID is provided, else skip.
			if ( isset( $args['connect_id'] ) ) {
				$wpdb->delete(
					$wpdb->prefix . $this->table_name,
					array(
						'id' => esc_attr( $args['connect_id'] ),
					)
				);
			}
		}

		/**
		 * Get the connect from database.
		 *
		 * @since 3.0
		 * @access public
		 * @param array $args The arguments.
		 * @return array connect data from database.
		 */
		public function get( $args ) {
			global $wpdb;

			// Check if connect ID is provided, else skip.
			if ( isset( $args['connect_id'] ) ) {
				$connect = $wpdb->get_results(
					$wpdb->prepare( 'SELECT * FROM `' . $wpdb->prefix . $this->table_name . '` WHERE `id` = %d', (int) $args['connect_id'] )
				);
			}

			$connect = ( ! empty( $connect ) && isset( $connect[0] ) ) ? $connect[0] : array();

			if ( ! empty( $connect ) ) {
				$connect->connect_data     = json_decode( base64_decode( $connect->connect_data ), true );
				$connect->connect_settings = json_decode( base64_decode( $connect->connect_settings ), true );
			}

			return $connect;
		}

		/**
		 * Get all connects from database.
		 *
		 * @since 3.0
		 * @access public
		 * @return object Connect data from database.
		 */
		public function get_all() {
			global $wpdb;

			return $wpdb->get_results(
				$wpdb->prepare( 'SELECT * FROM `' . $wpdb->prefix . $this->table_name . '` WHERE %s', 1 )
			);
		}

		/**
		 * Get all connects from database for the specified user.
		 *
		 * @since 3.0
		 * @access public
		 * @return object Connect data from database.
		 */
		public function get_user_connects( $user_email ) {
			global $wpdb;

			return $wpdb->get_results(
				$wpdb->prepare( 'SELECT * FROM `' . $wpdb->prefix . $this->table_name . '` WHERE `connect_settings` LIKE %s', '%"user_email":"' . $user_email . '"%' )
			);
		}
	}
}
