<?php
/**
 * Add application including triggers and actions to the applications.
 *
 * @access public
 * @since 1.0
 * @param string $slug        Application slug.
 * @param array  $application Application settings.
 * @return void
 */
function flowmattic_add_application( $slug, $application ) {
	if ( '' !== $slug && ! empty( $application ) ) {
		FlowMattic_Applications::add_application( $slug, $application );
	}
}

/**
 * Add custom app including triggers and actions to the applications.
 *
 * @access public
 * @since 3.0
 * @param array $custom_app Custom application settings.
 * @return void
 */
function flowmattic_add_custom_app( $custom_app ) {
	if ( ! empty( $custom_app ) ) {
		$custom_app->needs_connect = true;
		FlowMattic_Custom_Apps::add_custom_app( $custom_app );
	}
}

/**
 * Add connect for external app.
 *
 * @access public
 * @since 3.0
 * @param string $slug     Application slug.
 * @param array  $settings Connect settings.
 * @return void
 */
function flowmattic_add_connect( $slug, $settings ) {
	if ( '' !== $slug && ! empty( $settings ) ) {
		FlowMattic_Connects::add_connect( $slug, $settings );
	}
}

/**
 * Get stored connects for external app.
 *
 * @access public
 * @since 3.0
 * @param string $slug Connect slug.
 * @return array
 */
function flowmattic_get_connects( $slug = 'all' ) {
	return FlowMattic_Connects::get_connect( $slug );
}

/**
 * Generate random string.
 *
 * @access public
 * @since 1.0
 * @param int $length Application settings.
 * @return string
 */
function flowmattic_random_string( $length = 10 ) {
	$characters        = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
	$characters_length = strlen( $characters );
	$random_string     = '';

	for ( $i = 0; $i < $length; $i++ ) {
		$random_string .= $characters[ wp_rand( 0, $characters_length - 1 ) ];
	}

	return $random_string;
}

/**
 * Convert array to associative array based on the key and value pairs.
 *
 * @access public
 * @since 1.0
 * @param array  $ref_array Reference array.
 * @param string $ref_key   Reference array key.
 * @param array  $array     Array to be processed.
 * @return array
 */
function flowmattic_recursive_array( $ref_array, $ref_key, $array ) {
	if ( is_array( $array ) || is_object( $array ) ) {
		$array = (array) $array;

		foreach ( $array as $key => $value ) {
			$key         = str_replace( array( '*', chr( 0 ) ), '', $key );
			$new_ref_key = $ref_key . '_' . $key;

			if ( is_array( $value ) || is_object( $value ) ) {
				$value = (array) $value;
				if ( isset( $value[0] ) && ( is_array( $value[0] ) || is_object( $value[0] ) ) ) {
					$ref_array[ $new_ref_key ] = wp_json_encode( $value );
				} else {
					$ref_array = flowmattic_recursive_array( $ref_array, $new_ref_key, $value );
				}
			} else {
				$ref_array[ $new_ref_key ] = ( $value ) ? trim( $value ) : $value;
			}
		}
	}

	return $ref_array;
}

/**
 * Get contact form 7 forms list.
 *
 * @since 1.0
 * @access public
 * @return $forms Contact form 7 forms list.
 */
function flowmattic_get_contact_form_list() {
	$args = array(
		'post_type'      => 'wpcf7_contact_form',
		'posts_per_page' => -1,
		'post_status'    => 'publish',
	);

	$cf7_forms = array(
		'any' => esc_attr__( 'Any Form', 'flowmattic' ),
	);

	// @codingStandardsIgnoreLine
	if ( $data = get_posts( $args ) ) {
		foreach ( $data as $key ) {
			$cf7_forms[ $key->ID ] = $key->post_title;
		}
	}

	return $cf7_forms;
}

/**
 * Get WPForms forms list.
 *
 * @since 1.0
 * @access public
 * @return $forms WPForms forms list.
 */
function eewpb_get_wpform_list() {
	$args = array(
		'post_type'      => 'wpforms',
		'posts_per_page' => -1,
		'post_status'    => 'publish',
	);

	$wpforms = array(
		'any' => esc_attr__( 'Any Form', 'flowmattic' ),
	);

	// @codingStandardsIgnoreLine
	if ( $data = get_posts( $args ) ) {
		foreach ( $data as $key ) {
			$wpforms[ $key->ID ] = $key->post_title;
		}
	}

	return $wpforms;
}

/**
 * Get attachment values.
 *
 * @since 1.0
 * @access public
 * @param int $attachment_id Attachment ID.
 * @return array Attachment details.
 */
function flowmattic_get_attachment( $attachment_id ) {

	$attachment = get_post( $attachment_id );

	return array(
		'title'       => $attachment->post_title,
		'src'         => $attachment->guid,
		'caption'     => $attachment->post_excerpt,
		'description' => $attachment->post_content,
		'alt'         => get_post_meta( $attachment->ID, '_wp_attachment_image_alt', true ),
		'href'        => get_permalink( $attachment->ID ),
	);
}

/**
 * Decode the HTML entities for the given string.
 *
 * @since 1.0
 * @access public
 * @param string $to_decode String to decode.
 * @return string HTML entities decoded string.
 */
function flowmattic_decode_html_entities( $to_decode ) {
	if ( is_array( $to_decode ) ) {
		// If this is array, look through it and decode html entities if any.
		foreach ( $to_decode as $key => $value ) {
			$to_decode[ $key ] = html_entity_decode( $value );
		}
	} else {
		$to_decode = html_entity_decode( $to_decode );
	}

	return $to_decode;
}

/**
 * Get the workflow from database and extract the capture data from all steps.
 *
 * @since 1.0
 * @access public
 * @param string $workflow_id Workflow ID.
 * @return array All steps captured data.
 */
function flowmattic_get_workflow_captures( $workflow_id ) {
	$workflow_captures = array();

	$args = array(
		'workflow_id' => $workflow_id,
	);

	$workflow = wp_flowmattic()->workflows_db->get( $args );

	if ( ! empty( $workflow ) ) {
		$steps = ( ! is_array( $workflow ) ) ? json_decode( $workflow->workflow_steps, true ) : $workflow['workflow_steps'];

		foreach ( $steps as $key => $step ) {
			if ( isset( $step['capturedData'] ) ) {
				$workflow_captures[ $step['stepID'] ] = $step['capturedData'];
			}

			// In case the application is not set, skip the loop.
			if ( ! isset( $step['application'] ) ) {
				continue;
			}

			if ( 'router' === $step['application'] ) {
				$router_steps = $step['routerSteps'];
				$step_ids     = array();

				foreach ( $router_steps as $route_letter => $route_actions ) {
					$route_key = $key;
					$tag       = 'route' . $route_letter;

					foreach ( $route_actions as $rkey => $route_action ) {
						++$route_key;

						if ( ! isset( $route_action['stepID'] ) ) {
							continue;
						}

						$capture   = isset( $route_action['capturedData'] ) ? $route_action['capturedData'] : array();
						$route_tag = $tag . '.' . $route_action['application'] . $route_key;

						// Set the workflow capture.
						$workflow_captures[ $route_action['stepID'] ] = $capture;

						// Set the step id for internal task.
						$step_ids[] = array(
							'tag'    => $route_tag,
							'stepID' => $route_action['stepID'],
						);
					}
				}

				// Set the router as workflow capture.
				$workflow_captures['routers'][ $step['stepID'] ] = $step_ids;
			}
		}

		return $workflow_captures;
	} else {
		return array();
	}
}

/**
 * Get an array of all user data
 *
 * @since 1.0
 * @access public
 * @param  string|int $user_id User ID.
 * @return array merged array of user meta and data
 */
function flowmattic_get_user_data( $user_id ) {
	$user_data = (array) get_userdata( $user_id )->data;
	$user_meta = array_map(
		function( $item ) {
			return $item[0];
		},
		(array) get_user_meta( $user_id )
	);

	return array_merge( $user_data, $user_meta );
}

/**
 * Get the authentication data for the specified application and workflow.
 *
 * @since 1.0
 * @access public
 * @param  string $application Application slug.
 * @param  string $workflow_id Workflow ID.
 * @return array Authentication data for the application.
 */
function flowmattic_get_auth_data( $application, $workflow_id ) {
	$authentication_data = get_option( 'flowmattic_auth_data', array() );

	if ( isset( $authentication_data[ $workflow_id ] ) && ! empty( $authentication_data[ $workflow_id ] ) ) {
		if ( isset( $authentication_data[ $workflow_id ][ $application ] ) ) {
			return $authentication_data[ $workflow_id ][ $application ];
		}
	}

	return array();
}

/**
 * Update the authentication data for the specified application and workflow.
 *
 * @since 1.0
 * @access public
 * @param  string $application Application slug.
 * @param  string $workflow_id Workflow ID.
 * @param  array  $auth_data   Authentication data.
 * @return void.
 */
function flowmattic_update_auth_data( $application, $workflow_id, $auth_data ) {
	$authentication_data = get_option( 'flowmattic_auth_data', array() );

	if ( isset( $authentication_data[ $workflow_id ] ) && ! empty( $authentication_data[ $workflow_id ] ) ) {
		if ( isset( $authentication_data[ $workflow_id ][ $application ] ) ) {
			$authentication_data[ $workflow_id ][ $application ] = $auth_data;
			update_option( 'flowmattic_auth_data', $authentication_data, false );
		}
	}
}

/**
 * Output the license activated form.
 *
 * @since 1.0
 * @access public
 * @param Object $license License object.
 * @return void
 */
function flowmattic_license_activated_form( $license ) {
	?>
<form method="post" action="<?php echo esc_url( admin_url( 'admin-post.php' ) ); ?>">
	<input type="hidden" name="action" value="FlowMattic_el_deactivate_license"/>
	<div class="el-license-container">
		<p><h4 class="el-license-title fw-bold"><?php esc_html_e( 'License Info', 'flowmattic' ); ?></h4></p>
		<ul class="el-license-info list-group">
			<li class="list-group-item m-0">
				<div class="d-flex">
					<span class="el-license-info-title w-50"><?php esc_html_e( 'Status', 'flowmattic' ); ?></span>
					<?php if ( $license->is_valid ) : ?>
						<span class="el-license-valid badge bg-success d-inline-flex align-items-center"><?php esc_html_e( 'Valid', 'flowmattic' ); ?></span>
					<?php else : ?>
						<span class="el-license-valid badge bg-danger d-inline-flex align-items-center"><?php esc_html_e( 'Invalid', 'flowmattic' ); ?></span>
					<?php endif; ?>
				</div>
			</li>
			<li class="list-group-item m-0">
				<div class="d-flex">
					<span class="el-license-info-title w-50"><?php esc_html_e( 'License Type', 'flowmattic' ); ?></span>
					<?php echo $license->license_title; ?>
				</div>
			</li>
			<li class="list-group-item m-0">
				<div class="d-flex">
					<span class="el-license-info-title w-50"><?php esc_html_e( 'License Valid Till', 'flowmattic' ); ?></span>
					<?php
					if ( false !== strpos( $license->license_title, 'Lifetime' ) ) {
						echo 'LIFETIME';
					} else {
						echo date_i18n( 'd F, Y', strtotime( $license->expire_date ) );
					}

					if ( ! empty( $license->expire_renew_link ) ) {
						?>
						<a target="_blank" class="el-blue-btn" href="<?php echo $license->expire_renew_link; ?>">Renew</a>
						<?php
					}
					?>
				</div>
			</li>
			<li class="list-group-item m-0">
				<div class="d-flex">
					<span class="el-license-info-title w-50"><?php esc_html_e( 'Support Valid Till', 'flowmattic' ); ?></span>
					<?php
					if ( false !== strpos( $license->license_title, 'Lifetime' ) ) {
						echo 'LIFETIME';
					} else {
						echo date_i18n( 'd F, Y', strtotime( $license->support_end ) );
					}

					if ( ! empty( $license->support_renew_link ) ) {
						?>
						<a target="_blank" class="el-blue-btn" href="<?php echo $license->support_renew_link; ?>">Renew</a>
						<?php
					}
					?>
				</div>
			</li>
			<li class="list-group-item m-0">
				<div class="d-flex">
					<span class="el-license-info-title w-50"><?php esc_html_e( 'Your License Key', 'flowmattic' ); ?></span>
					<span class="el-license-key fw-bold"><?php echo esc_attr( substr( $license->license_key, 0, 9 ) . 'XXXXXXXX-XXXXXXXX' . substr( $license->license_key, -9 ) ); ?></span>
				</div>
			</li>
		</ul>
		<div class="el-license-active-btn">
			<?php wp_nonce_field( 'el-license' ); ?>
			<p class="submit">
				<button type="submit" name="submit" id="submit" class="btn btn-danger d-inline-flex align-items-center">
					<svg width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg" data-reactroot="">
						<path stroke-linejoin="round" stroke-linecap="round" stroke-width="1" stroke="#fff" d="M8 10V5C8 3.34315 9.34315 2 11 2H13C14.6569 2 16 3.34315 16 5V6"></path>
						<rect stroke-linejoin="round" stroke-linecap="round" stroke-width="1" stroke="#fff" fill="none" rx="2" height="12" width="14" y="10" x="5"></rect>
						<path stroke-linejoin="round" stroke-linecap="round" stroke-width="1" stroke="#fff" d="M12 17L12 15"></path>
						<circle stroke-width="1" stroke="#fff" fill="none" r="1" cy="18" cx="12"></circle>
					</svg>
					<span class="ps-2"><?php echo __( 'Deactivate License', 'flowmattic' ); ?></span>
				</button>
			</p>
		</div>
	</div>
</form>
	<?php
}

/**
 * Output the license form to enter license key and activate.
 *
 * @since 1.0
 * @access public
 * @return void
 */
function flowmattic_license_form() {
	$license_key = get_option( 'flowmattic_license_Key', '' );
	$license_key = ( '' !== $license_key ) ? esc_attr( substr( $license_key, 0, 9 ) . 'XXXXXXXX-XXXXXXXX' . substr( $license_key, -9 ) ) : 'xxxxxxxx-xxxxxxxx-xxxxxxxx-xxxxxxxx';
	?>
<form method="post" action="<?php echo esc_url( admin_url( 'admin-post.php' ) ); ?>">
	<input type="hidden" name="action" value="FlowMattic_el_activate_license"/>
	<div class="el-license-container">
		<p><h4 class="el-license-title fw-bold"><?php esc_html_e( 'Activate License', 'flowmattic' ); ?></h4></p>
		<?php
		$license_message = get_option( 'flowmattic_license_message', '' );
		if ( $license_message ) {
			?>
			<div class="alert alert-danger d-flex align-items-center" role="alert">
				<svg class="me-2" width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg" data-reactroot="">
					<path fill="#842029" d="M12 2L2 22H22L12 2Z" undefined="1"></path><path fill="#842029" d="M12 19C12.2761 19 12.5 18.7761 12.5 18.5C12.5 18.2239 12.2761 18 12 18C11.7239 18 11.5 18.2239 11.5 18.5C11.5 18.7761 11.7239 19 12 19Z" undefined="1"></path>
					<path stroke-linejoin="round" stroke-linecap="round" stroke-miterlimit="10" stroke-width="1" stroke="#fff" d="M12 19C12.2761 19 12.5 18.7761 12.5 18.5C12.5 18.2239 12.2761 18 12 18C11.7239 18 11.5 18.2239 11.5 18.5C11.5 18.7761 11.7239 19 12 19Z"></path>
					<path stroke-linejoin="round" stroke-linecap="round" stroke-miterlimit="10" stroke-width="1" stroke="#fff" d="M12 10V15"></path>
				</svg>
				<div><?php echo esc_html( $license_message ); ?></div>
			</div>
			<?php
			delete_option( 'flowmattic_license_message' );
		}
		?>
		<div class="p-3 border rounded mb-3 bg-light">
			<div class="el-license-field mb-3">
				<label class="form-label" for="flowmattic_license_key"><?php esc_html_e( 'License code', 'flowmattic' ); ?></label>
				<input type="text" class="regular-text code form-control" name="flowmattic_license_key" size="50" placeholder="<?php echo $license_key; ?>" required="required">
			</div>
			<div class="el-license-field mb-3">
				<label class="form-label" for="flowmattic_license_key"><?php esc_html_e( 'Email Address', 'flowmattic' ); ?></label>
				<?php
				$purchase_email = get_option( 'flowmattic_license_email', get_bloginfo( 'admin_email' ) );
				?>
				<input type="text" class="regular-text code form-control" name="el_license_email" size="50" value="<?php echo $purchase_email; ?>" placeholder="" required="required" aria-describedby="emailHelp">
				<div id="emailHelp" class="form-text"><small><?php esc_html_e( 'Email used to purchase the plugin', 'flowmattic' ); ?></small></div>
			</div>
			<div class="el-license-active-btn">
				<?php wp_nonce_field( 'el-license' ); ?>
				<button type="submit" name="submit" id="submit" class="btn btn-primary d-inline-flex align-items-center">
					<svg width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg" data-reactroot="">
						<path stroke-linejoin="round" stroke-linecap="round" stroke-width="1" stroke="#fff" d="M8 5C8 3.34315 9.34315 2 11 2H13C14.6569 2 16 3.34315 16 5V10H8V5Z"></path>
						<rect stroke-linejoin="round" stroke-linecap="round" stroke-width="1" stroke="#fff" fill="none" rx="2" height="12" width="14" y="10" x="5"></rect>
						<path stroke-linejoin="round" stroke-linecap="round" stroke-width="1" stroke="#fff" d="M12 17L12 15"></path>
						<circle stroke-width="1" stroke="#fff" fill="none" r="1" cy="18" cx="12"></circle>
					</svg>
					<span class="ps-2"><?php echo __( 'Activate License', 'flowmattic' ); ?></span>
				</button>
			</div>
		</div>
		<p class="fs-6 fw-bold mb-3"><?php esc_html_e( 'Enter your license key here, to activate the product, and get full feature updates and premium support.', 'flowmattic' ); ?></p>
		<ul class="list-group list-group-flush m-0">
			<li class="list-group-item ps-1"><svg class="me-1" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg" data-reactroot="">
<path fill="#333" d="M3.29289 3.29289C3.68342 2.90237 4.31658 2.90237 4.70711 3.29289L12.7071 11.2929C13.0976 11.6834 13.0976 12.3166 12.7071 12.7071L4.70711 20.7071C4.31658 21.0976 3.68342 21.0976 3.29289 20.7071C2.90237 20.3166 2.90237 19.6834 3.29289 19.2929L10.5858 12L3.29289 4.70711C2.90237 4.31658 2.90237 3.68342 3.29289 3.29289Z" clip-rule="evenodd" fill-rule="evenodd" undefined="1"></path>
<path fill="#333" d="M11.2929 3.29289C11.6834 2.90237 12.3166 2.90237 12.7071 3.29289L20.7071 11.2929C21.0976 11.6834 21.0976 12.3166 20.7071 12.7071L12.7071 20.7071C12.3166 21.0976 11.6834 21.0976 11.2929 20.7071C10.9024 20.3166 10.9024 19.6834 11.2929 19.2929L18.5858 12L11.2929 4.70711C10.9024 4.31658 10.9024 3.68342 11.2929 3.29289Z" clip-rule="evenodd" fill-rule="evenodd" undefined="1"></path>
</svg><?php echo sprintf( __( 'You will find your license key at the license tab in <a href="%s" target="_blank">account page</a> on our website.', 'flowmattic' ), 'https://flowmattic.com/account/#licenses' ); ?></li>
			<li class="list-group-item ps-1"><svg class="me-1" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg" data-reactroot="">
<path fill="#333" d="M3.29289 3.29289C3.68342 2.90237 4.31658 2.90237 4.70711 3.29289L12.7071 11.2929C13.0976 11.6834 13.0976 12.3166 12.7071 12.7071L4.70711 20.7071C4.31658 21.0976 3.68342 21.0976 3.29289 20.7071C2.90237 20.3166 2.90237 19.6834 3.29289 19.2929L10.5858 12L3.29289 4.70711C2.90237 4.31658 2.90237 3.68342 3.29289 3.29289Z" clip-rule="evenodd" fill-rule="evenodd" undefined="1"></path>
<path fill="#333" d="M11.2929 3.29289C11.6834 2.90237 12.3166 2.90237 12.7071 3.29289L20.7071 11.2929C21.0976 11.6834 21.0976 12.3166 20.7071 12.7071L12.7071 20.7071C12.3166 21.0976 11.6834 21.0976 11.2929 20.7071C10.9024 20.3166 10.9024 19.6834 11.2929 19.2929L18.5858 12L11.2929 4.70711C10.9024 4.31658 10.9024 3.68342 11.2929 3.29289Z" clip-rule="evenodd" fill-rule="evenodd" undefined="1"></path>
</svg><?php esc_html_e( 'Enter your license key and the email used to purchase the plugin.', 'flowmattic' ); ?></li>
			<li class="list-group-item ps-1"><svg class="me-1" width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg" data-reactroot="">
<path fill="#333" d="M3.29289 3.29289C3.68342 2.90237 4.31658 2.90237 4.70711 3.29289L12.7071 11.2929C13.0976 11.6834 13.0976 12.3166 12.7071 12.7071L4.70711 20.7071C4.31658 21.0976 3.68342 21.0976 3.29289 20.7071C2.90237 20.3166 2.90237 19.6834 3.29289 19.2929L10.5858 12L3.29289 4.70711C2.90237 4.31658 2.90237 3.68342 3.29289 3.29289Z" clip-rule="evenodd" fill-rule="evenodd" undefined="1"></path>
<path fill="#333" d="M11.2929 3.29289C11.6834 2.90237 12.3166 2.90237 12.7071 3.29289L20.7071 11.2929C21.0976 11.6834 21.0976 12.3166 20.7071 12.7071L12.7071 20.7071C12.3166 21.0976 11.6834 21.0976 11.2929 20.7071C10.9024 20.3166 10.9024 19.6834 11.2929 19.2929L18.5858 12L11.2929 4.70711C10.9024 4.31658 10.9024 3.68342 11.2929 3.29289Z" clip-rule="evenodd" fill-rule="evenodd" undefined="1"></path>
</svg><?php esc_html_e( 'Once the license is activated, you\'ll be able to receive automatic updates and unlock premium features.', 'flowmattic' ); ?></li>
		</ul>
	</div>
</form>
	<?php
}

/**
 * Action to activate the license.
 *
 * @since 1.0
 * @access public
 * @return void
 */
function flowmattic_action_activate_license() {

	check_admin_referer( 'el-license' );

	$license_key   = ! empty( $_POST['flowmattic_license_key'] ) ? $_POST['flowmattic_license_key'] : '';
	$license_email = ! empty( $_POST['el_license_email'] ) ? $_POST['el_license_email'] : '';

	update_option( 'flowmattic_license_key', $license_key, false );
	update_option( 'flowmattic_license_email', $license_email, false );
	update_option( '_site_transient_update_plugins', '', false );

	// Delete transient for integrations.
	delete_transient( 'flowmattic_integrations' );

	wp_safe_redirect( admin_url( 'admin.php?page=flowmattic-license' ) );
}

/**
 * Action to deactivate the license.
 *
 * @since 1.0
 * @access public
 * @return void
 */
function flowmattic_action_deactivate_license() {
	check_admin_referer( 'el-license' );
	$message = '';
	if ( FlowMattic_Updater::RemoveLicenseKey( FLOWMATTIC_PLUGIN_FILE, $message ) ) {
		update_option( 'flowmattic_license_key', '', false );
		update_option( '_site_transient_update_plugins', '', false );
	}
	wp_safe_redirect( admin_url( 'admin.php?page=flowmattic-license' ) );
}

add_action( 'admin_post_FlowMattic_el_deactivate_license', 'flowmattic_action_deactivate_license' );
add_action( 'admin_post_FlowMattic_el_activate_license', 'flowmattic_action_activate_license' );

/**
 * Redirect to dashboard on plugin activation.
 *
 * @since 1.0
 * @access public
 * @param string $plugin Plugin slug.
 * @return void
 */
function flowmattic_activation_redirect( $plugin ) {
	if ( plugin_basename( FLOWMATTIC_PLUGIN_FILE ) === $plugin ) {
		exit( wp_safe_redirect( admin_url( 'admin.php?page=flowmattic' ) ) );
	}
}
add_action( 'activated_plugin', 'flowmattic_activation_redirect' );

/**
 * Listen to the custom actions for workflow.
 *
 * @access public
 * @since 1.0
 * @param string $workflow_id Workflow ID.
 * @param array  $data        Action data.
 * @return void
 */
function flowmattic_custom_action_trigger( $workflow_id, $data ) {
	$workflow_live_id = get_option( 'webhook-capture-live', false );
	$submited_data    = $data;
	$simple_entry     = array();

	foreach ( $data as $key => $value ) {
		if ( is_array( $value ) ) {
			$simple_entry = flowmattic_recursive_array( $simple_entry, $key, $value );
		} else {
			$simple_entry[ $key ] = $value;
		}
	}

	$submited_data = $simple_entry;

	if ( $workflow_live_id ) {
		update_option( 'webhook-capture-' . $workflow_live_id, $submited_data, false );
		delete_option( 'webhook-capture-live' );

		// Do not execute workflow if capture data in process.
		return;
	}

	// Run the workflow.
	$flowmattic_workflow = new FlowMattic_Workflow();
	$flowmattic_workflow->run( $workflow_id, $submited_data );
}
add_action( 'flowmattic_trigger_workflow', 'flowmattic_custom_action_trigger', 99, 2 );

/**
 * Register custom cron schedule for hourly.
 *
 * @access public
 * @since 1.0.3
 * @param array $schedules Cron schedules.
 * @return array
 */
function flowmattic_custom_hourly_cron_schedule( $schedules ) {
	$schedules['flowmattic_hourly'] = array(
		'interval' => 3590,
		'display'  => __( 'Once every hour' ),
	);

	for ( $i = 1; $i < 60; $i++ ) {
		$interval = 'flowmattic_every_' . $i . '_minutes';

		// Adds per minute to the existing schedules.
		$schedules[ $interval ] = array(
			'interval' => ( $i * 60 ),
			'display'  => 'Every ' . $i . ' Minutes ( FlowMattic )',
		);
	}

	return $schedules;
}

add_filter( 'cron_schedules', 'flowmattic_custom_hourly_cron_schedule' );

/* Create FlowMattic Workflow Manager User Role */
if ( get_role( 'contributor' ) ) {
	add_role(
		'flowmattic_workflow_manager', // System name of the role.
		__( 'FlowMattic - Workflow Manager' ), // Display name of the role.
		array(
			'read'          => true,
			'edit_posts'    => true,
			'publish_posts' => true,
			'delete_posts'  => true,
		)
	);
}

/**
 * Upgrade the administrator Role
 *
 * @access public
 * @since 1.3.0
 * @return void
 */
function flowmattic_admin_level_up() {
	// If using ajax, don't execute!
	if ( ! defined( 'DOING_AJAX' ) ) {
		// Retrieve the  Administrator role.
		$role = get_role( 'administrator' );

		// Let's add a set  of new capabilities we want Administrators to have.
		$role->add_cap( 'manage_workflows' );

		// Retrieve the  Workflow manager role.
		$role = get_role( 'flowmattic_workflow_manager' );

		// Let's add a set  of new capabilities we want Workflow manager to have.
		$role->add_cap( 'manage_workflows' );
	}
}
add_action( 'admin_init', 'flowmattic_admin_level_up' );

/**
 * Parse webhook data for WhatsApp.
 *
 * @access public
 * @since 1.4.0
 * @param array $webhook_data Webhook data from WhatsApp webhook.
 * @return array
 */
function flowmattic_whatsapp_webhook_response( $webhook_data ) {
	$json_decode_data = json_decode( $webhook_data, true );
	$response_data    = array();

	foreach ( $json_decode_data[0]['value'] as $key => $value ) {
		if ( 'messages' === $key ) {
			$response_data = $value[0];

			$response_data['text'] = $response_data['text']['body'];

			if ( isset( $response_data['button'] ) ) {
				$response_data['button_payload'] = ( isset( $response_data['button'][0]['payload'] ) ) ? $response_data['button'][0]['payload'] : '';
				$response_data['button_text']    = ( isset( $response_data['button'][0]['text'] ) ) ? $response_data['button'][0]['text'] : '';
				unset( $response_data['button'] );
			}
		}
	}

	return $response_data;
}

/**
 * Ajax to handle workflow export file download.
 *
 * @access public
 * @since 1.4.0
 * @return void
 */
function flowmattic_export_workflow() {
	check_ajax_referer( 'flowmattic_workflow_nonce', 'workflow_nonce' );

	$workflow_id = isset( $_POST['workflowID'] ) ? $_POST['workflowID'] : '';

	// Get the workflow.
	$args     = array(
		'workflow_id' => $workflow_id,
	);
	$workflow = wp_flowmattic()->workflows_db->get( $args );

	$workflow_steps = json_decode( $workflow->workflow_steps, true );

	foreach ( $workflow_steps as $key => $step ) {
		// Remove captured data from export file.
		unset( $step['capturedData'] );

		foreach ( $step as $data_key => $data_value ) {
			if ( ! is_array( $data_value ) ) {
				$data_value        = stripslashes( $data_value );
				$step[ $data_key ] = $data_value;
			}
		}

		$workflow_steps[ $key ] = $step;
	}

	$data = array(
		'workflow_name'     => $workflow->workflow_name,
		'workflow_steps'    => $workflow_steps,
		'workflow_settings' => json_decode( $workflow->workflow_settings, true ),
	);

	nocache_headers();
	header( 'Content-Type: application/json; charset=utf-8' );
	header( 'Content-Disposition: attachment; filename=flowmattic-workflow-' . $workflow_id . '.json' );
	header( 'Expires: 0' );

	echo wp_json_encode( $data );

	die();
}

add_action( 'wp_ajax_flowmattic_export_workflow', 'flowmattic_export_workflow' );

/**
 * Ajax to handle workflow import.
 *
 * @access public
 * @since 1.4.0
 * @return void
 */
function flowmattic_import_workflow() {
	check_ajax_referer( 'flowmattic_workflow_nonce', 'workflow_nonce' );

	$workflow_data = isset( $_POST['workflowData'] ) ? $_POST['workflowData'] : '';

	$data = array(
		'workflow_id'       => flowmattic_random_string(),
		'workflow_name'     => $workflow_data['workflow_name'] . '_IMPORTED',
		'workflow_steps'    => is_array( $workflow_data['workflow_steps'] ) ? $workflow_data['workflow_steps'] : json_decode( $workflow_data['workflow_steps'], true ),
		'workflow_settings' => is_array( $workflow_data['workflow_settings'] ) ? $workflow_data['workflow_settings'] : json_decode( $workflow_data['workflow_settings'], true ),
	);

	// Set the imported workflow as draft.
	$data['workflow_settings']['status'] = 'off';

	// Set the workflow access to blank.
	$data['workflow_settings']['user_email'] = '';

	// Set the imported time.
	$data['workflow_settings']['time'] = date_i18n( 'd-m-Y h:i:s A' );

	$workflow_db = wp_flowmattic()->workflows_db;
	$status      = $workflow_db->insert( $data );

	$reply = array(
		'status' => $status,
	);

	echo wp_json_encode( $reply );

	die();
}

add_action( 'wp_ajax_flowmattic_import_workflow', 'flowmattic_import_workflow' );

/**
 * Ajax to handle workflow template import.
 *
 * @access public
 * @since 3.1.0
 * @return void
 */
function flowmattic_import_workflow_template() {
	check_ajax_referer( 'flowmattic_workflow_nonce', 'workflow_nonce' );

	$template_id   = isset( $_POST['template_id'] ) ? $_POST['template_id'] : '';
	$template_name = isset( $_POST['template_name'] ) ? $_POST['template_name'] : '';
	$license_key   = get_option( 'flowmattic_license_key', '' );

	$site_url    = site_url();
	$server_host = 'https://flowmattic.com/workflows/';

	$args = array(
		'license'     => $license_key,
		'site'        => $site_url,
		'method'      => 'downloadWorkflow',
		'template_id' => $template_id,
		'nonce'       => wp_create_nonce( 'workflow-templates' ),
	);

	$request       = wp_remote_get( $server_host . '?' . http_build_query( $args ) );
	$request       = wp_remote_retrieve_body( $request );
	$workflow_data = json_decode( $request, true );
	$workflow_id   = flowmattic_random_string();

	$data = array(
		'workflow_id'       => $workflow_id,
		'workflow_name'     => ( '' !== $template_name ) ? $template_name : $workflow_data['workflow_name'] . '_IMPORTED',
		'workflow_steps'    => is_array( $workflow_data['workflow_steps'] ) ? $workflow_data['workflow_steps'] : json_decode( $workflow_data['workflow_steps'], true ),
		'workflow_settings' => is_array( $workflow_data['workflow_settings'] ) ? $workflow_data['workflow_settings'] : json_decode( $workflow_data['workflow_settings'], true ),
	);

	// Set the imported workflow as draft.
	$data['workflow_settings']['status'] = 'off';

	// Set the workflow access to blank.
	$data['workflow_settings']['user_email'] = '';

	// Set the imported time.
	$data['workflow_settings']['time'] = date_i18n( 'd-m-Y h:i:s A' );

	$workflow_db = wp_flowmattic()->workflows_db;
	$status      = $workflow_db->insert( $data );

	$reply = array(
		'status'      => $status,
		'workflow_id' => $workflow_id,
	);

	echo wp_json_encode( $reply );

	die();
}

add_action( 'wp_ajax_flowmattic_import_workflow_template', 'flowmattic_import_workflow_template' );

/**
 * Ajax to handle workflow clone.
 *
 * @access public
 * @since 2.0
 * @return void
 */
function flowmattic_clone_workflow() {
	check_ajax_referer( 'flowmattic_workflow_nonce', 'workflow_nonce' );

	$workflow_id = isset( $_POST['workflowID'] ) ? $_POST['workflowID'] : '';

	// Get the workflow.
	$args     = array(
		'workflow_id' => $workflow_id,
	);
	$workflow = wp_flowmattic()->workflows_db->get( $args );

	$workflow_steps = json_decode( $workflow->workflow_steps, true );

	foreach ( $workflow_steps as $key => $step ) {
		foreach ( $step as $data_key => $data_value ) {
			if ( ! is_array( $data_value ) ) {
				$data_value        = stripslashes( $data_value );
				$step[ $data_key ] = $data_value;
			}
		}

		$workflow_steps[ $key ] = $step;
	}

	$data = array(
		'workflow_id'       => flowmattic_random_string(),
		'workflow_name'     => $workflow->workflow_name . '_CLONED',
		'workflow_steps'    => $workflow_steps,
		'workflow_settings' => json_decode( $workflow->workflow_settings, true ),
	);

	// Set the cloneed workflow as draft.
	$data['workflow_settings']['status'] = 'off';

	// Set the cloned time.
	$data['workflow_settings']['time'] = date_i18n( 'd-m-Y h:i:s A' );

	$workflow_db = wp_flowmattic()->workflows_db;
	$status      = $workflow_db->insert( $data );

	$reply = array(
		'status' => $status,
	);

	echo wp_json_encode( $reply );

	die();
}

add_action( 'wp_ajax_flowmattic_clone_workflow', 'flowmattic_clone_workflow' );

/**
 * Check if the integration is disabled for user level.
 *
 * @access public
 * @since 1.0
 * @param array $application Application slug.
 * @return bool
 */
function flowmattic_is_app_disabled( $application ) {
	$settings = get_option( 'flowmattic_settings', array() );

	if ( isset( $settings[ 'disable-app-' . $application ] ) && $application === $settings[ 'disable-app-' . $application ] ) {
		return true;
	} else {
		return false;
	}
}

/**
 * Register add_action for each workflow if it is using plugin actions as trigger.
 *
 * @access public
 * @since 2.0
 * @return void
 */
function flowmattic_execute_hooks() {
	$workflow_hooks = get_option( 'flowmattic_workflow_hooks', array() );
	if ( ! empty( $workflow_hooks ) ) {
		foreach ( $workflow_hooks as $workflow_id => $capture_hook ) {
			add_action( $capture_hook, 'flowmattic_capture_plugin_action_data', 10, 10 );
		}
	}
}

/**
 * Execute the action set in the plugin actions trigger..
 *
 * @access public
 * @since 2.0
 * @return array
 */
function flowmattic_capture_plugin_action_data() {
	$num_args = func_num_args();
	$args     = func_get_args();

	if ( ! $num_args ) {
		return false;
	}

	$final_capture = array();

	$live_workflow_id = get_option( 'webhook-capture-live', false );

	// Get all workflow hooks registered.
	$workflow_hooks = get_option( 'flowmattic_workflow_hooks', array() );

	// Loop through workflow hooks and if current action is fired, execute the workflow.
	if ( ! empty( $workflow_hooks ) ) {
		foreach ( $workflow_hooks as $workflow_id => $capture_hook ) {
			// Check if the current action is fired.
			if ( did_action( $capture_hook ) ) {
				if ( 1 === $num_args ) {
					$final_capture = is_array( $args ) ? $args[0] : $args;
				} else {
					for ( $i = 0; $i < $num_args; $i++ ) {
						$final_capture[ 'arg-' . $i ] = $args[ $i ];
					}
				}

				// Check if live capturing is in place, and that matches the capturing workflow.
				if ( $workflow_id === $live_workflow_id ) {

					update_option( 'webhook-capture-' . $workflow_id, $final_capture, false );
					delete_option( 'webhook-capture-live' );

					// If it is in capturing mode, skip the workflow execution by returning data here.
					return $final_capture;
				}

				// Run the workflow.
				$flowmattic_workflow = new FlowMattic_Workflow();
				$flowmattic_workflow->run( $workflow_id, $final_capture );
			}
		}
	}
}

add_action( 'init', 'flowmattic_execute_hooks' );

// Remove FluentCRM global search script from the workflows editor.
if ( isset( $_GET['page'] ) && 'flowmattic-workflows' === $_GET['page'] ) {
	add_filter( 'fluentcrm_disable_global_search', '__return_true' );
}

/**
 * Save the files from mailhook to WP.
 *
 * @access public
 * @since 2.2.0
 * @param string $filename     Filename with extension.
 * @param string $base64_file  Base64 encoded file.
 * @param string $content_type Content type.
 */
function flowmattic_import_file_from_mailhook( $filename, $base64_file, $content_type ) {

	// Upload dir.
	$upload_dir  = wp_upload_dir();
	$upload_path = str_replace( '/', DIRECTORY_SEPARATOR, $upload_dir['path'] ) . DIRECTORY_SEPARATOR;

	$file_decode = base64_decode( $base64_file ); // @codingStandardsIgnoreLine

	// Save the file in the uploads directory.
	$upload_file = file_put_contents( $upload_path . $filename, $file_decode ); // @codingStandardsIgnoreLine

	$attachment = array(
		'post_mime_type' => $content_type,
		'post_title'     => $filename,
		'post_content'   => '',
		'post_status'    => 'inherit',
		'guid'           => $upload_dir['url'] . '/' . $filename,
	);

	// Insert attachment for use in WP.
	$file_id = wp_insert_attachment( $attachment, $upload_dir['path'] . '/' . $filename );

	// Return the file attachment post ID.
	return array(
		'file_id'      => $file_id,
		'file_name'    => $filename,
		'file_url'     => wp_get_attachment_url( $file_id, 'full' ),
		'content_type' => $content_type,
	);
}

/**
 * Parse the CSV data.
 *
 * @access public
 * @since 2.2.0
 * @param string $encoded_csv_data Base64 encoded CSV file data.
 */
function flowmattic_parse_csv( $encoded_csv_data ) {
	$csv_content = trim( base64_decode( $encoded_csv_data ) ); // @codingStandardsIgnoreLine
	$csv_data    = array_map( 'str_getcsv', explode( "\n", $csv_content ) );

	try {
		// Read csv headers.
		$headers = $csv_data[0];
		unset( $csv_data[0] );

		// Initialize CSV array.
		$csv_json = array();

		foreach ( $csv_data as $key => $row ) {
			$csv_json[] = array_combine( $headers, $row );
		}

		return array(
			'data' => wp_json_encode( $csv_json ),
		);
	} catch ( Error $e ) {
		return array(
			'data'  => wp_json_encode( $csv_data ),
			'error' => $e->getMessage(),
		);
	}
}

/**
 * Save the files from mailhook to WP.
 *
 * @access public
 * @since 2.2.0
 * @param string $email_text  Email in text.
 */
function flowmattic_parse_emails_from_mailhook( $email_text ) {
	// Test string for checking email.
	$test_patt = "/(?:[a-z0-9!#$%&'*+=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+=?^_`{|}~-]+)*|\"(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21\x23-\x5b\x5d-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])*\")@(?:(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?|\[(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?|[a-z0-9-]*[a-z0-9]:(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21-\x5a\x53-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])+)\])/";

	// Convert text to lower case.
	$email_text = strtolower( $email_text );

	// Comapare using preg_match_all() method.
	preg_match_all( $test_patt, $email_text, $valid );

	$parsed_emails = array();
	if ( ! empty( $valid ) ) {
		$emails = array_unique( $valid[0] );

		foreach ( $emails as $email ) {
			$parsed_emails[] = array( $email );
		}

		return array(
			'array' => $parsed_emails,
			'list'  => implode( ',', $emails ),
		);
	}

	return array();
}

/**
 * Get the attachment absolute path from its url
 *
 * @access public
 * @since 2.2.0
 * @param string $url the attachment url to get its absolute path.
 *
 * @return bool|string It returns the absolute path of an attachment.
 */
function flowmattic_attachment_url_to_path( $url ) {
	$parsed_url = wp_parse_url( $url );
	if ( empty( $parsed_url['path'] ) ) {
		return false;
	}

	$file = ABSPATH . ltrim( $parsed_url['path'], '/' );
	if ( file_exists( $file ) ) {
		return $file;
	}

	return false;
}

/**
 * Remove the unwanted CSS and JS scripts from 3rd party plugins on FlowMattic admin pages to avoid conflicts.
 *
 * @access public
 * @since 3.0
 * @param string $hook Current page hook prefix.
 *
 * @return void
 */
function flowmattic_dequeue_unwanted_scripts( $hook ) {
	if ( false !== strpos( $hook, 'flowmattic' ) ) {
		// Remove other conflicting CSS.
		wp_dequeue_style( 'eazydocs-admin-global' );

		// Remove other conflicting JS.
		$scripts = array(
			'ezd-notify-review',
			'tutor-admin',
			'tutor-script',
			'bootstrap',
			'jquery-modal',
			'wp-reset',
			'wp-backbone',
			'itsec-core-admin-notices',
			'happyfiles',
			'7b997e0a187e5239b28537747fd73a3e',
		);

		foreach ( $scripts as $handle ) {
			wp_dequeue_script( $handle );
		}

		// Remove jQuery modal scripts.
		wp_deregister_style( 'jquery-modal' );

		// Disable the emoji's.
		remove_action( 'wp_head', 'print_emoji_detection_script', 7 );
		remove_action( 'admin_print_scripts', 'print_emoji_detection_script' );
		remove_action( 'wp_print_styles', 'print_emoji_styles' );
		remove_action( 'admin_print_styles', 'print_emoji_styles' );
		remove_filter( 'the_content_feed', 'wp_staticize_emoji' );
		remove_filter( 'comment_text_rss', 'wp_staticize_emoji' );
		remove_filter( 'wp_mail', 'wp_staticize_emoji_for_email' );
	}
}
add_action( 'admin_enqueue_scripts', 'flowmattic_dequeue_unwanted_scripts' );

/**
 * Custom alternative to download_url to avoid conflicts with server config.
 *
 * @access public
 * @since 3.0
 * @param string $url File URL to download.
 *
 * @return string File.
 */
function flowmattic_download_url( $url ) {
	// Temporary file to store the downloaded content.
	$tmp_file = wp_tempnam( $url );

	// Attempt to download using wp_remote_get.
	$response = wp_remote_get( $url, array( 'timeout' => 300 ) );

	if ( ! is_wp_error( $response ) && 200 === wp_remote_retrieve_response_code( $response ) ) {
		$body = wp_remote_retrieve_body( $response );
		file_put_contents( $tmp_file, $body );
		return $tmp_file;
	}

	// If wp_remote_get fails, use cURL as a fallback.
	if ( function_exists( 'curl_init' ) ) {
		$ch = curl_init();
		$fp = fopen( $tmp_file, 'wb' );

		curl_setopt( $ch, CURLOPT_URL, $url );
		curl_setopt( $ch, CURLOPT_HEADER, 0 );
		curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, 1 );
		curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );
		curl_setopt( $ch, CURLOPT_SSL_VERIFYHOST, 0 );
		curl_setopt( $ch, CURLOPT_FILE, $fp );
		curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 10 );
		curl_setopt( $ch, CURLOPT_TIMEOUT, 300 );

		$success = curl_exec( $ch );

		curl_close( $ch );
		fclose( $fp );

		if ( $success ) {
			return $tmp_file;
		}
	}

	// If both methods fail, return an error.
	return new WP_Error( 'download_failed', __( 'Download failed.' ) );
}

/**
 * Get the integrations.
 *
 * @access public
 * @since 3.0
 * @param string $type Requested integration type.
 *
 * @return object Current live integrations.
 */
function flowmattic_get_integrations( $type = 'integrations' ) {
	$all_integrations = get_transient( 'flowmattic_integrations' );
	$license_key      = get_option( 'flowmattic_license_key', '' );

	if ( false === $all_integrations && '' !== $license_key ) {
		$site_url    = site_url();
		$server_host = 'https://updates.flowmattic.com/integration/';

		$args = array(
			'license' => $license_key,
			'site'    => $site_url,
			'type'    => $type,
		);

		$request          = wp_remote_get( $server_host . '?' . http_build_query( $args ) );
		$request          = wp_remote_retrieve_body( $request );
		$all_integrations = json_decode( $request );

		set_transient( 'flowmattic_integrations', $all_integrations, HOUR_IN_SECONDS * 24 );
	}

	return $all_integrations;
}

/**
 * Get the integrations.
 *
 * @access public
 * @since 3.1.0
 *
 * @return object Get workflow templates.
 */
function flowmattic_get_workflow_templates() {
	$all_workflow_templates = get_transient( 'flowmattic_workflow_templates' );
	$license_key            = get_option( 'flowmattic_license_key', '' );

	if ( false === $all_workflow_templates && '' !== $license_key ) {
		$site_url    = site_url();
		$server_host = 'https://flowmattic.com/workflows/';

		$args = array(
			'license' => $license_key,
			'site'    => $site_url,
			'method'  => 'getWorkflows',
			'nonce'   => wp_create_nonce( 'workflow-templates' ),
		);

		$request                = wp_remote_get( $server_host . '?' . http_build_query( $args ) );
		$request                = wp_remote_retrieve_body( $request );
		$all_workflow_templates = json_decode( $request, true );

		set_transient( 'flowmattic_workflow_templates', $all_workflow_templates, HOUR_IN_SECONDS * 24 );
	}

	return $all_workflow_templates;
}

/**
 * Helper function to get the time formats.
 *
 * @access public
 * @since 3.0
 *
 * @return array All available time formats.
 */
function flowmattic_get_time_formats() {
	return array(
		'D M d H:i:s O Y' => 'ddd MMM DD HH:mm:ss Z YYYY (Sun Jan 22 23:04:05 -0000 2023)',
		'D m d H:i'       => 'ddd MM DD HH:mm (Sun Jan 22 23:04)',
		'F d Y H:i:s'     => 'MMMM DD YYYY HH:mm:ss (January 22 2023 23:04:05)',
		'F d Y'           => 'MMMM DD YYYY (January 22 2023)',
		'M d Y'           => 'MMM DD YYYY (Jan 22 2023)',
		'Y-m-dTH:i:sO'    => 'YYYY-MM-DDTHH:mm:ssZ (2023-01-22T23:04:05-0000)',
		'Y-m-d H:i:s O'   => 'YYYY-MM-DD HH:mm:ss Z (2023-01-22 23:04:05 -0000)',
		'Y-m-d'           => 'YYYY-MM-DD (2023-01-22)',
		'm-d-Y'           => 'MM-DD-YYYY (01-22-2023)',
		'm/d/Y'           => 'MM/DD/YYYY (01/22/2023)',
		'm/d/y'           => 'MM/DD/YY (01/22/23)',
		'd-m-Y'           => 'DD-MM-YYYY (22-01-2023)',
		'd/m/Y'           => 'DD/MM/YYYY (22/01/2023)',
		'd/m/y'           => 'DD/MM/YY (22/01/23)',
		'Y-m-d H:i:s'     => 'YYYY-MM-DD HH:mm:ss (2023-11-05 13:08:16)',
		'd/m/Y h:i:s A'   => 'DD/MM/YYYY hh:mm:ss A (09/12/2023 05:30:11 PM)',
		'd-m-Y H:i:s'     => 'DD-MM-YYYY HH:mm:ss (09-12-2023 15:30:11)',
		'd-m-Y h:i:s A'   => 'DD-MM-YYYY hh:mm:ss A (09-12-2023 5:30:11 AM)',
		'Y-m-d G:i:s'     => 'YYYY-MM-DD H:M:S (2023-07-28 7:7:0)',
		'H:i'             => 'HH:mm (13:08)',
		'U'               => 'x (1684059642) Microsecond',
	);
}

/**
 * Replace a specific value with another value in a nested array.
 *
 * @access public
 * @since 3.0
 * @param array  $nested_array  The input nested array to search and replace the value in.
 * @param string $search_value  The value to search for in the nested array.
 * @param mixed  $replace_value The value to replace the search value with. This can be any data type.
 * @return array The updated nested array with the search value replaced by the replace value.
 */
function flowmattic_dynamic_tag_values( $nested_array, $search_value, $replace_value ) {
	// $nested_array = flowmattic_stripslashes_deep( $nested_array );
	$json_string = stripslashes( wp_json_encode( $nested_array ) );

	// Check if the search value exists in the JSON-encoded array.
	if ( strpos( $json_string, $search_value ) === false ) {
		// Search value not found, return the original array.
		return $nested_array;
	}
	foreach ( $nested_array as $key => $value ) {
		if ( is_array( $value ) ) {
			$nested_array[ $key ] = flowmattic_dynamic_tag_values( $value, $search_value, $replace_value );
		} elseif ( is_array( $replace_value ) ) {
			if ( $value === $search_value ) {
				$nested_array[ $key ] = $replace_value;
			}
		} else {
			$nested_array[ $key ] = str_replace( $search_value, $replace_value, $value );
		}
	}

	return $nested_array;
}

/**
 * Perform a deep stripslashes operation on array items
 *
 * @access public
 * @since 3.0.1
 * @param mixed $value Value to be cleaned from slashes.
 *
 * @return mixed Cleaned value.
 */
function flowmattic_stripslashes_deep( $value ) {
	$value = is_array( $value ) ? array_map( 'flowmattic_stripslashes_deep', $value ) : stripslashes( $value );

	return $value;
}

/**
 * Retrieves a value from a nested array by index.
 *
 * @access public
 * @since 3.0.2
 * @param array  $array The array to search.
 * @param string $index The index to search for.
 *
 * @return mixed|null The value of the found index or null if not found.
 */
function flowmattic_get_value_by_index( $array, $index ) {
	// Check if the index exists in the array.
	if ( isset( $array[ $index ] ) ) {
		return $array[ $index ];
	}

	// If the index does not exist in the array, look for it in the nested arrays.
	foreach ( $array as $value ) {
		if ( is_array( $value ) ) {
			$result = flowmattic_get_value_by_index( $value, $index );
			if ( $result !== null ) {
				return $result;
			}
		} elseif ( is_array( json_decode( $value, true ) ) ) {
			$value  = json_decode( $value, true );
			$result = flowmattic_get_value_by_index( $value, $index );
			if ( $result !== null ) {
				return $result;
			}
		}
	}

	// Return null if the index is not found.
	return null;
}
